@@ -141,7 +141,6 @@ Constant RUNTIME_ERRORS = 2;
 Constant RTE_MINIMUM = 0;
 Constant RTE_NORMAL = 1;
 Constant RTE_VERBOSE = 2;
-Constant TM_NOT_PUNY;
 #Endif;
 #Iftrue RUNTIME_ERRORS > RTE_MINIMUM;
 Constant TM_ERR = "^[Talk_menu error #";
@@ -244,6 +243,7 @@ Constant TM_ADD_BEFORE_AND_AFTER 4; ! Can be used directly after subject
 
 Global talk_menu_talking = false;
 Global talk_menu_multi_mode = true;
+Global clr_talk_menu = CLR_CURRENT;
 
 [ _TMPrintMsg p_msg p_no_newline;
 	if(metaclass(p_msg) == Routine) {
@@ -294,33 +294,7 @@ Global talk_menu_multi_mode = true;
 			_index = _index - 2;
 		}
 		_val = p_array-->_index;
-		if(_val == TM_END) {
-			if(_find_topic < 29) {
-				! Signal failure, as the topic was not found
-				if(p_value == 1) rtrue;
-				rfalse;
-			}
-			if(_stash_array) {
-				p_array = _stash_array;
-				_index = 0;
-				_stash_array = 0;
-			} else {
-				! The topic wasn't found, or we are in multi mode
-				if(p_value == 1) rtrue; ! When trying to read a status, reaching the end means failure 
-				return _success; ! The topic wasn't found, or we are in multi mode
-			}
-		} else if(_val == TM_MAYBE_ADD_LIST) {
-			if(_find_topic < 29) {
-				! Signal failure, as the topic was not found
-				if(p_value == 1) rtrue;
-				rfalse;
-			}
-			_index = _index + 2;
-			_curr_id = p_array-->_index;
-			_stash_array = p_array + _index + _index;
-			p_array = _curr_id;
-			_index = -1;
-		} else if(_val == TM_INACTIVE or TM_ACTIVE or TM_STALE) {
+		if(_val == TM_INACTIVE or TM_ACTIVE or TM_STALE) {
 			if(_find_topic < 29) {
 				if(_find_topic-- == 1) jump _tm_found_topic;
 				continue;
@@ -350,6 +324,34 @@ Global talk_menu_multi_mode = true;
 
 				}
 			}
+		} else if(_val == TM_MAYBE_ADD_LIST or TM_END) {
+			if(_val == TM_MAYBE_ADD_LIST) {
+				if(_find_topic < 29) {
+					! Signal failure, as the topic was not found
+					if(p_value == 1) rtrue;
+					rfalse;
+				}
+				_index = _index + 2;
+				_curr_id = p_array-->_index;
+				_stash_array = p_array + _index + _index;
+				p_array = _curr_id;
+				_index = -1;
+			} else  { ! _val == TM_END
+				if(_find_topic < 29) {
+					! Signal failure, as the topic was not found
+					if(p_value == 1) rtrue;
+					rfalse;
+				}
+				if(_stash_array) {
+					p_array = _stash_array;
+					_index = 0;
+					_stash_array = 0;
+				} else {
+					! The topic wasn't found, or we are in multi mode
+					if(p_value == 1) rtrue; ! When trying to read a status, reaching the end means failure 
+					return _success; ! The topic wasn't found, or we are in multi mode
+				}
+			}
 		}
 	}
 ];
@@ -418,8 +420,13 @@ Array TenDashes static -> "----------";
 Array _TMLines --> 10;
 
 #Ifv5;
-[ RunTalk p_npc _array _i _j _n _val _height _width _offset _count _more _has_split _add_msg _stash_array;
+#Ifdef PUNYINFORM_MAJOR_VERSION;
+[ RunTalk p_npc _array _i _j _n _val _height _width _offset _count _more _has_split _add_msg _stash_array _old_fg;
 #Ifnot;
+[ RunTalk p_npc _array _i _j _n _val _height _width _offset _count _more _has_split _add_msg _stash_array;
+#Endif;
+#Ifnot;
+! Target is z3 
 [ RunTalk p_npc _array _i _j _n _val _offset _count _more _add_msg _stash_array;
 #Endif;
 	talk_menu_talking = true;
@@ -507,6 +514,12 @@ Array _TMLines --> 10;
 				DrawStatusLine();
 				@set_window 1;
 				@set_cursor 2 1;
+#Ifdef PUNYINFORM_MAJOR_VERSION;
+				if(clr_on) {
+					_old_fg = clr_fg;
+					ChangeFgColour(clr_talk_menu);
+				}
+#Endif;
 				_TMPrintMsg(TM_MSG_TALK_ABOUT_WHAT);
 			}
 			print "  ", _count % 10, ": ";
@@ -571,6 +584,12 @@ Array _TMLines --> 10;
 		_TMPrintMsg(_j, true);
 	}
 	new_line;
+#Ifdef PUNYINFORM_MAJOR_VERSION;
+#Ifv5;
+	if(clr_on)
+		ChangeFgColour(_old_fg);
+#Endif;
+#Endif;
 
 	! Ask player to choose a line to say
 ._askAgain;
@@ -578,7 +597,7 @@ Array _TMLines --> 10;
 	_j = 0;
 	while(_j == 0) {
 #IfV5;
-		@read_char -> _val;
+		@read_char 1 -> _val;
 
 		if(_val == 13 or 'q' or 'Q' or 'x' or 'X') {
 			@set_window 0;
@@ -739,10 +758,10 @@ Array _TMLines --> 10;
 		@erase_window 1;
 		@split_window 0;
 		@set_window 0;
-		#Ifdef TM_NOT_PUNY;
-			gg_statuswin_cursize = 0;
-		#Ifnot;
+		#Ifdef PUNYINFORM_MAJOR_VERSION;
 			statusline_current_height = 0;
+		#Ifnot;
+			gg_statuswin_cursize = 0;
 		#Endif;
 	}
 #Endif;
@@ -751,12 +770,12 @@ Array _TMLines --> 10;
 ];
 
 [ TalkSub;
-#Ifdef TM_NOT_PUNY;
-   if (noun==player) { L__M(##Tell, 1, noun); rtrue; }
-   if (~~(noun provides talk_array)) { L__M(##Show, 2, second); rtrue; }
-#Ifnot;
+#Ifdef PUNYINFORM_MAJOR_VERSION;
    if (noun==player) { PrintMsg(MSG_TELL_PLAYER); rtrue; }
    if (~~(noun provides talk_array)) { second = noun; PrintMsg(MSG_SHOW_DEFAULT); rtrue; }
+#Ifnot;
+   if (noun==player) { L__M(##Tell, 1, noun); rtrue; }
+   if (~~(noun provides talk_array)) { L__M(##Show, 2, second); rtrue; }
 #Endif;
    RunTalk(noun);
    AfterRoutines();
