@@ -95,10 +95,10 @@ Verb 'insert'
 #Ifdef OPTIONAL_FLEXIBLE_INVENTORY;
 Verb 'inventory' 'i//'
 	* -> Inv
-	* 'tall'/'wide' -> Inv;
+	* 'tall'/'wide'                             -> Inv;
 #Ifnot;
 Verb 'inventory' 'i//'
-	* -> Inv;
+	*                                           -> Inv;
 #Endif;
 
 Verb 'jump'
@@ -145,7 +145,7 @@ Verb 'put' 'place'
 Verb 'read'
 	* noun                                      -> Examine
 	* 'about' topic 'in' noun                   -> Consult reverse
-	* topic 'in' noun                           -> Consult;
+	* topic 'in' noun                           -> Consult reverse;
 
 Verb 'remove'
 	* worn                                      -> Disrobe
@@ -218,7 +218,7 @@ Verb 'turn' 'rotate' 'screw' 'twist' 'unscrew'
 Verb 'unlock'
 	* noun 'with' held                          -> Unlock;
 
-Verb 'wait' 'z'
+Verb 'wait' 'z//'
 	*                                           -> Wait;
 
 Verb 'wear'
@@ -288,13 +288,15 @@ Verb 'wear'
 	return MSG_DRINK_NOTHING_SUITABLE;
 ];
 
-[ DropSub _p;
+[ DropSub;
 	if(noun notin player) return MSG_DROP_NOT_HOLDING;
 	if(ImplicitDisrobeIfWorn(noun)) rtrue;
-	_p = parent(player);
-	!if(_p ~= location) <<Insert noun _p>>;
 	move noun to parent(player);
 	give noun moved;
+#Ifndef OPTIONAL_NO_DARKNESS;
+	if(location == thedark)
+		scope_modified = true;
+#Endif;
 	run_after_routines_msg = MSG_DROP_DROPPED;
 ];
 
@@ -307,30 +309,48 @@ Verb 'wear'
 	run_after_routines_msg = MSG_EAT_DEFAULT;
 ];
 
+#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
+[ _CheckDoorDirTo p_door;
+#Ifdef OPTIONAL_SIMPLE_DOORS;
+	if(p_door.&door_dir == 0 || (p_door.&door_to == 0 && (p_door.&found_in == 0 || p_door.#found_in ~= 4))) {
+		_RunTimeError(ERR_OBJECT_HASNT_PROPERTY, p_door);
+		rtrue;
+	}
+#Ifnot;
+	if(p_door.&door_to == 0 || p_door.&door_dir == 0) {
+		_RunTimeError(ERR_OBJECT_HASNT_PROPERTY, p_door);
+		rtrue;
+	}
+#EndIf;
+];
+#EndIf;
+
+
+[ DoorDir p_door _door_dir;
+#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
+	_CheckDoorDirTo(p_door);
+#EndIf;
+#IfDef OPTIONAL_SIMPLE_DOORS;
+	if(p_door.#door_dir > 2) {
+		! This is a Simple Door, where door_dir is an array
+		if(real_location == p_door.&found_in-->1)
+			_door_dir = 1;
+		return p_door.&door_dir-->_door_dir;
+	} else {
+#EndIf;
+		! Normal Inform door
+		_door_dir = p_door.door_dir;
+		if(UnsignedCompare(_door_dir, top_object) > 0)
+			return p_door.door_dir();
+		return _door_dir;
+#IfDef OPTIONAL_SIMPLE_DOORS;
+	}
+#EndIf;
+];
+
 [ EnterSub _door_dir;
 	if(noun has door) {
-#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
-		if(noun.&door_dir == 0) { ! door_to will be checked by Go action anyway
-			_RunTimeError(ERR_OBJECT_HASNT_PROPERTY, noun);
-			rtrue;
-		}
-#EndIf;
-#IfDef OPTIONAL_SIMPLE_DOORS;
-		if(noun.#door_dir > 2) {
-			! This is a Simple Door, where door_dir is an array
-			if(real_location == noun.&found_in-->1)
-				_door_dir = 1;
-			_door_dir = noun.&door_dir-->_door_dir;
-		} else {
-#EndIf;
-			! Normal Inform door
-			_door_dir = noun.door_dir;
-			if(UnsignedCompare(_door_dir, top_object) > 0) {
-				_door_dir = noun.door_dir();
-			}
-#IfDef OPTIONAL_SIMPLE_DOORS;
-		}
-#EndIf;
+		_door_dir = DoorDir(noun);
 		! Convert to fake object
 		_door_dir = DirPropToFakeObj(_door_dir);
 		<<Go _door_dir>>;
@@ -496,7 +516,7 @@ Verb 'wear'
 ! 7: Second isn't supporter
 ! 8: Check if second is full
 ! 9: Default (success) message
-Array _InsertMessages -->
+Array _InsertMessages static -->
 	MSG_INSERT_ALREADY
 	MSG_INSERT_ITSELF
 	MSG_INSERT_NOT_OPEN
@@ -597,7 +617,7 @@ Array _InsertMessages -->
 ! 7: Second isn't supporter
 ! 8: Check if second is full
 ! 9: Default (success) message
-Array _PutOnMessages -->
+Array _PutOnMessages static -->
 	MSG_PUTON_ALREADY
 	MSG_PUTON_ITSELF
 	0
@@ -615,8 +635,9 @@ Array _PutOnMessages -->
 ];
 
 [ RemoveSub _i;
+	if(noun == player)
+		<<Exit second>>;
 	_i = parent(noun);
-	if (_i has container && _i hasnt open) { PrintMsg(MSG_REMOVE_CLOSED, _i); rtrue; }
 	if (_i ~= second) return MSG_REMOVE_NOT_HERE;
 	if(TryToTakeNoun() ~= false) rtrue;
 	action = ##Remove; if (AfterRoutines()) rtrue;
@@ -688,7 +709,7 @@ Array _PutOnMessages -->
 ];
 
 [ TakeSub;
-	if(TryToTakeNoun() ~= false) rtrue;
+	if(TryToTakeNoun(true) ~= false) rtrue;
 	run_after_routines_msg = MSG_TAKE_DEFAULT;
 ];
 
@@ -724,13 +745,24 @@ Array _PutOnMessages -->
 	return MSG_TOUCH_DEFAULT;
 ];
 
-[ TransferSub;
-	if(noun in second || (noun in parent(player) && selected_direction == d_to))
+[ TransferSub _work_to_do;
+	if(noun notin second) _work_to_do = true;
+	if(second == player) { <Take noun>; run_after_routines_msg = _work_to_do; rtrue; }
+	if(_work_to_do == false || (noun in parent(player) && second == Directions && selected_direction == d_to))
 		return MSG_TRANSFER_ALREADY;
 	if(noun notin player && TryToTakeNoun() == true) rtrue;
-	if (second has supporter) <<PutOn noun second>>;
-	if (second == Directions && selected_direction == d_to) <<Drop noun>>;
-	<Insert noun second>;
+	if (second == Directions && selected_direction == d_to) {
+		<Drop noun>;
+		if(noun in location) run_after_routines_msg = 1;
+	}
+	else {
+		if (second has supporter) 
+			<PutOn noun second>;
+		else 
+			<Insert noun second>;
+		if(noun in second)
+			run_after_routines_msg = 1;
+	}
 ];
 
 [ TurnSub;
@@ -787,8 +819,7 @@ Verb 'buy' 'purchase'
 	* noun                                      -> Buy;
 
 Verb 'consult'
-	* noun 'about' topic                        -> Consult
-	* noun 'on' topic                           -> Consult;
+	* noun 'about'/'on' topic                   -> Consult;
 
 Verb 'empty'
 	* noun                                      -> Empty
@@ -885,9 +916,10 @@ Verb 'yes' 'y//'
 	<EmptyT noun FAKE_D_OBJ>;
 ];
 
-[ EmptyTSub _i _recipient;
+[ EmptyTSub _i _n _obj _recipient;
 	if(noun == second) return MSG_EMPTY_WOULDNT_ACHIEVE;
-	if(noun has container && noun hasnt open) {
+	if(noun hasnt container) { PrintMsg(MSG_EMPTY_NOT_CONTAINER, noun); rtrue; }
+	if(noun hasnt open) {
 		PrintMsg(MSG_EMPTY_IS_CLOSED, noun);
 		rtrue;
 	}
@@ -895,13 +927,19 @@ Verb 'yes' 'y//'
 		_recipient = DirPropToFakeObj(selected_direction);
 	else
 		_recipient = second;
-	_i = child(noun);
-	if(_i == 0) { PrintMsg(MSG_EMPTY_ALREADY_EMPTY, noun); rtrue; }
-	while(_i ~= 0) {
-		if(keep_silent == 0) print (name) _i, ": ";
-		<Transfer _i _recipient>;
-		if(_i in noun || _i in player) rtrue;
-		_i = child(noun);
+	_obj = child(noun);
+	if(_obj == 0) { PrintMsg(MSG_EMPTY_ALREADY_EMPTY, noun); rtrue; }
+	while(_obj ~= 0 && _n<MAX_SCOPE) {
+		empty_arr-->_n = _obj;
+		_n++;
+		_obj = sibling(_obj);
+	}
+	for(_i=0: _i<_n: _i++) {
+		_obj = empty_arr-->_i;
+		if(_obj in noun) {
+			if(keep_silent == 0) print (name) _obj, ": ";
+			<Transfer _obj _recipient>;
+		}
 	}
 	run_after_routines_msg = 1;
 ];
@@ -1095,7 +1133,7 @@ Verb meta 'quit' 'q//'
 		PANum(_score_sum);
 		PrintMsg(MSG_FULLSCORE_ACTIONS);
 	}
-	@new_line;
+	new_line;
 	PANum(score);
 	return MSG_FULLSCORE_END;
 ];
@@ -1202,6 +1240,8 @@ Verb meta 'quit' 'q//'
 	#EndIf;
 	#IfDef Headline;
 		print (string) Headline;
+	#IfNot;
+		new_line;
 	#EndIf;
 #EndIf;
 	print "Release ", (0-->1) & $03ff, " / Serial number ";
@@ -1210,27 +1250,23 @@ Verb meta 'quit' 'q//'
 	inversion;
 	print " PunyInform v", PUNYINFORM_MAJOR_VERSION, (char) '.', PUNYINFORM_MINOR_VERSION;
 	if(PUNYINFORM_PATCH_VERSION)
-		print ".", PUNYINFORM_PATCH_VERSION;
+		print (char) '.', PUNYINFORM_PATCH_VERSION;
 #Ifdef PUNYINFORM_VERSION_SUFFIX;
 	print (string) PUNYINFORM_VERSION_SUFFIX;
 #EndIf;
-	_i = 0;
+	print (char) ' ';
 #IfDef STRICT_MODE;
 	#IfV5;
-	print " S";
-	_i = 1;
+	print (char) 'S';
 	#EndIf;
 #EndIf;
 #IfDef DEBUG;
-	if(_i == 0) print " ";
-	print "D";
-	_i = 1;
+	print (char) 'D';
 #EndIf;
 #IfTrue RUNTIME_ERRORS > 0;
-	if(_i == 0) print " ";
-	print "R";
+	print (char) 'R';
 #EndIf;
-	@new_line;
+	new_line;
 ];
 
 [ VersionSub;
@@ -1290,48 +1326,46 @@ Verb meta 'verify'
 ];
 
 #Ifndef NO_PLACES;
-[ PlacesSub i j k;
-	print "You have visited: ";
-	objectloop(i has visited) if(parent(i) == 0) j++;
-	objectloop(i has visited) {
-		if(parent(i) == 0) {
-			print (name) i; k++;
-			if (k == j) { print ".^"; return; }
-			if (k == j-1) print " and ";
-			else          print ", ";
+[ PlacesSub _i;
+	print "You have visited:^";
+	objectloop(_i has visited) {
+		if(parent(_i) == 0 && _RoomLike(_i)) {
+#Ifndef OPTIONAL_NO_DARKNESS;
+			if(_i == thedark) continue;
+#Endif;
+			print "- ", (name) _i; new_line;
 		}
 	}
 ];
 
-[ ObjectsSub i j f;
+[ ObjectsSub _i _j _f;
 	print "Objects you have handled: ";
-	objectloop (i has moved) {
-		j = parent(i);
-		if(j) {
-			if(f == 0) @new_line;
-			f = 1;
-			print "- ", (the) i, "   ";
-			if (j == player) {
-				if (i has worn) {
-					print "(worn)";
-				} else {
-					print "(held)";
-				}
-			} else if(j has animate) print "(given away)";
-			else if(j has visited) print "(in ", (name) j, ")";
-			else if(j has container) print "(inside ", (the) j, ")";
-			else if(j has supporter) print "(on ", (the) j, ")";
-			else if(j has enterable) print "(in ", (the) j, ")";
-			else print "(lost)";
-			@new_line;
-		}
+	objectloop (_i has moved && _ObjectLike(_i)) {
+		_j = parent(_i);
+		if(_f == 0) new_line;
+		_f++;
+		print "- ", (the) _i, "   ";
+		if (_j == player) {
+			if (_i has worn) {
+				print "(worn)";
+			} else {
+				print "(held)";
+			}
+		} else if(_j == 0) print "(lost)";
+		else if(_j has animate) print "(given away)";
+		else if(_j has visited && parent(_j)==0) print "(in ", (name) _j, ")";
+		else if(_j has container) print "(inside ", (the) _j, ")";
+		else if(_j has supporter) print "(on ", (the) _j, ")";
+		else if(_j has enterable) print "(in ", (the) _j, ")";
+		else print "(lost)";
+		new_line;
 	}
-	if(f == 0) "none.";
+	if(_f == 0) "none.";
 ];
 #Endif; ! NO_PLACES
 
 [ ScriptOnSub;
-	transcript_mode = ((HDR_GAMEFLAGS-->0) & 1);
+	transcript_mode = (HDR_GAMEFLAGS-->0) & 1;
 	if (transcript_mode) "Transcripting is already on.";
 	@output_stream 2;
 	if (((HDR_GAMEFLAGS-->0) & 1) == 0) "Attempt to begin transcript failed.";
@@ -1364,8 +1398,12 @@ Verb meta 'verify'
 ! ---------------------
 
 #IfDef DEBUG;
-Verb meta 'pronoun' 'pronouns'
+Verb meta 'pronoun'
 	*                                           -> Pronouns;
+#IfV5;
+Verb meta 'pronouns'
+	*                                           -> Pronouns;
+#EndIf;
 
 Verb meta 'random'
 	*                                           -> RandomSeed
@@ -1381,11 +1419,22 @@ Verb meta 'purloin'
 
 Verb meta 'tree'
 	*                                           -> Tree
+	* number                                    -> Tree
 	* noun                                      -> Tree;
 
+Verb meta 'forest'
+	*                                           -> Forest;
+
+Verb meta 'rooms'
+	*                                           -> Rooms
+	* topic                                     -> Rooms;
+
 Verb meta 'gonear'
 	* noun                                      -> GoNear;
 
+Verb meta 'goto'
+	* topic                                     -> Goto;
+
 Verb meta 'debug'
 	*                                           -> Debug
 	* 'reactive'                                -> Debug;
@@ -1410,9 +1459,35 @@ Global scope_cnt;
 [ GoNearSub _obj;
 	_obj = noun;
 	while(parent(_obj) ~= 0) _obj = parent(_obj);
+	if(_obj == noun) print_ret (The) noun, " is not in anything!";
 	PlayerTo(_obj);
 ];
 
+#Ifndef GOTOSUB_BUFFER_SIZE;
+Constant GOTOSUB_BUFFER_SIZE 80;
+#Endif;
+
+Array _GotoSubBuffer --> (1 + (GOTOSUB_BUFFER_SIZE + 1)/2); ! Add an extra word of constant has odd value
+
+[ GotoSub _obj;
+	if(consult_words == 1) {
+		_obj = TryNumber(consult_from);
+		if(_obj > 0) {
+			if(_RoomLike(_obj))
+				jump _gotoObj;
+			jump _not_a_room;
+		}
+	}
+	_obj = _GotoRoomsHelper(true);
+	if(_obj == 0)
+		jump _not_a_room;
+._gotoObj;
+	PlayerTo(_obj);
+	rtrue;
+._not_a_room;
+	"That doesn't seem to be a room.";
+];
+
 [ PronounsSub;
 	print "It: ", (name) itobj, "^Him: ", (name) himobj, "^Her: ", (name) herobj, "^Them: ", (name) themobj, "^";
 ];
@@ -1440,7 +1515,7 @@ Global scope_cnt;
 [ _ScopeSubHelper p_obj;
 	print scope_cnt++,": ", (a) p_obj, " (", p_obj, ")";
 	if(ObjectIsUntouchable(p_obj, true)) print " [untouchable]";
-	@new_line;
+	new_line;
 ];
 
 [ ScopeSub;
@@ -1449,31 +1524,143 @@ Global scope_cnt;
 	if(scope_cnt < 2) "Nothing in scope.^";
 ];
 
-[ TreeSub _obj _p;
-	_obj = noun;
-	if(_obj==0) _obj = real_location;
-	print (name) _obj;
-	_p = parent(_obj);
-	if(_p) {
-		print " (";
-		if(_p has supporter)
-			@print_char 'o';
-		else
-			@print_char 'i';
-		print "n ", (name) _p, ")";
-	}
-	@new_line;
-	TreeSubHelper(_obj, 1);
-];
-
-[TreeSubHelper p_parent p_indent _x _i;
+[ _TreeSubHelper p_parent p_indent _x _i;
 	objectloop(_x in p_parent) {
 		for(_i = 0 : _i < p_indent : _i++) print "  ";
-		print (name) _x, " (", _x, ")^";
-		if(child(_x)) TreeSubHelper(_x, p_indent + 1);
+		print (a) _x, " (", _x, ")^";
+		if(child(_x)) _TreeSubHelper(_x, p_indent + 1);
 	}
 ];
 
+Constant _REAL_LOCATION_TEXT " *** real_location ***";
+
+[ TreeSub p_real_location _p;
+	if(parsed_number > 0 && noun == parsed_number) {
+		if(parsed_number < Directions || parsed_number > top_object)
+			"That doesn't seem to be an object.";
+	}
+
+	if(noun==0) noun = real_location;
+	if(noun in nothing)
+		print (name) noun;
+	else
+		print (a) noun;
+	print " (", noun, ")";
+	_p = parent(noun);
+	if(_p) {
+		if(_p has supporter)
+			print " on";
+		else
+			print " in";
+		print " ~", (name) _p, "~ (", _p, ")";
+	}
+	if(noun == p_real_location) {
+#IfV5;
+		style bold;
+#Endif;
+		print (string) _REAL_LOCATION_TEXT;
+#IfV5;
+		style roman;
+#Endif;
+	}
+	new_line;
+	_TreeSubHelper(noun, 1);
+];
+
+[ ForestSub;
+	for(noun=Directions : noun<= top_object: noun++)
+		if(noun in nothing)
+			TreeSub(real_location);
+];
+
+[ _GotoRoomsHelper p_return_first _obj _first _i _j _k _m _n _t _count 
+		_first_typed_char _first_typed_char_upcase _last_start_pos;
+	! p_return_first = true: return the first matching room
+	! p_return_first = false: print all matching rooms
+	_t = _GotoSubBuffer + 2;
+	if(consult_from) {
+		_first = WordAddress(consult_from);
+		if(_first->0 == '*') _first++; ! Ignore '*' at start of search string
+		_i = consult_from + consult_words - 1;
+		_count = WordAddress(_i) + WordLength(_i) - _first;
+		if(_first->(_count-1) == '*') _count--;  ! Ignore '*' at end of search string
+		_first_typed_char = _first->0;
+		_first_typed_char_upcase = _first_typed_char;
+		if(_first_typed_char_upcase < 123 && _first_typed_char_upcase > 96) {
+			_first_typed_char_upcase = _first_typed_char_upcase - 32;
+		}
+		
+	}
+	_obj=Directions + 1;
+!	for(_obj=Directions + 1 : _obj<= top_object: _obj++)
+._check_next_obj;
+		if(_RoomLike(_obj)) {
+
+			if(_count > 0) {
+				@output_stream 3 _GotoSubBuffer;
+				print (name) _obj;
+				@output_stream -3;
+				_k = _GotoSubBuffer-->0;
+#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
+				if(_k > GOTOSUB_BUFFER_SIZE) {
+					_RunTimeError(ERR_BUFFER_OVERRUN, _obj);
+					rtrue;
+				}
+#Endif;
+				_last_start_pos = _k - _count;
+
+				! Check if search string is part of room name
+				_j = 0;
+				!for(_j=0:_j<=_last_start_pos:_j++) {
+._match_loop;
+					if(_t->_j == _first_typed_char or _first_typed_char_upcase) {
+						! Found a match for first character
+						if(_count == 1) jump _found_match;
+						_m = _count - 1;
+						_i = 1;
+						_n = _j + 1;
+!							for(_i=1,_n=_j+1:_i<_count:_i++,_n++) {
+._match_next_char;
+							_k = _t->_n;
+							if(_k < 91 && _k > 64)
+								_k = _k + 32;
+							if(_k ~= _first->_i) {
+								jump _leave_inner_match_loop;
+							}
+							_n++;
+							@inc_chk _i _m ?~_match_next_char;
+!							}
+						jump _found_match;
+._leave_inner_match_loop;
+					}
+					@inc_chk _j _last_start_pos ?~_match_loop;
+!				}
+				jump _end_of_obj_loop;
+			} 
+._found_match;		
+			if(p_return_first)
+				return _obj;
+			print (name) _obj, " (", _obj, ")";
+			if(_obj == real_location) {
+#Ifv5;
+				style bold;
+#Endif;
+				print (string) _REAL_LOCATION_TEXT;
+#Ifv5;
+				style roman;
+#Endif;
+			}
+			new_line;
+		}
+._end_of_obj_loop;
+		@inc_chk _obj top_object ?~_check_next_obj;
+	return false;
+];
+
+[ RoomsSub;
+	_GotoRoomsHelper();
+];
+
 #Ifdef OPTIONAL_MANUAL_REACTIVE;
 [ MayBeRoutine p_obj p_prop _val;
 	_val = p_obj.&p_prop;
@@ -1565,17 +1752,16 @@ Global scope_cnt;
 ];
 
 [ _ListObjsInOnMsg p_parent;
-	if(newline_flag)
-		print "^";
+	print "^";
 	if(p_parent has supporter) print "On "; else print "In ";
 	print (the) p_parent, " you can ";
 	if(also_flag) print "also ";
 	print "see ";
 ];
 
-[ Look _obj _top_ceil _ceil _describe_room
-	_you_can_see_1 _you_can_see_2 _desc_prop _last_level _action;
-	PrintMsg(MSG_LOOK_BEFORE_ROOMNAME);
+[ Look _obj _top_ceil _ceil _describe_room _you_can_see_1 _you_can_see_2 
+		_desc_prop _last_level _action;
+	if(input_action == ##Look) PrintMsg(MSG_LOOK_BEFORE_ROOMNAME);
 	if((lookmode == 1 && location hasnt visited) || lookmode == 2) _describe_room = true;
 #IfV5;
 	style bold;
@@ -1603,15 +1789,14 @@ Global scope_cnt;
 		}
 #EndIf;
 		_PrintObjName(location);
-	} else {
+	} else
 		print (The) _ceil;
-	}
 #IfV5;
 	style roman;
 #EndIf;
 #Ifndef OPTIONAL_NO_DARKNESS;
 	if(location == thedark) {
-		@new_line;
+		new_line;
 		PrintOrRun(location, description);
 		jump _EndOfLookRoutine;
 	}
@@ -1625,51 +1810,46 @@ Global scope_cnt;
 		print (the) _obj, ")";
 		_obj = parent(_obj);
 	}
+	new_line;
 	while(_ceil ~= player or 0) {
 		if(_describe_room) {
 			if(_ceil == location) {
-				@new_line;
 				PrintOrRun(_ceil, description);
 			} else if(_ceil.inside_description ~= 0 or NULL) {
-				@new_line;
+				if(_ceil ~= _top_ceil) new_line;
 				PrintOrRun(_ceil, inside_description);
 			}
-		} else if(_ceil == location)
-			@new_line;
+		}
 
 		also_flag = false;
 		! write intial and describe messages in a new paragraph
-		objectloop(_obj in _ceil && _obj hasnt scenery or concealed && _obj ~= player) {
-			give _obj workflag;
-			if(_obj.&describe) {
-				if(PrintOrRun(_obj, describe, 0)) {
-					give _obj ~workflag;
-					also_flag = true;
-					continue;
+		objectloop(_obj in _ceil)
+			if(_obj hasnt scenery or concealed && _obj ~= player) {
+				give _obj workflag;
+				if(_obj.&describe) {
+					if(PrintOrRun(_obj, describe, 0)) {
+						give _obj ~workflag;
+						also_flag = true;
+						continue;
+					}
 				}
-			}
-			if(_obj has container or door) {
-				if(_obj has open) {
-					_desc_prop = when_open;
-				} else {
+				if(_obj has container or door) {
 					_desc_prop = when_closed;
-				}
-			} else if(_obj has switchable) {
-				if(_obj has on) {
-					_desc_prop = when_on;
-				} else {
+					if(_obj has open)
+						_desc_prop = when_open;
+				} else if(_obj has switchable) {
 					_desc_prop = when_off;
+					if(_obj has on)
+						_desc_prop = when_on;
+				} else
+					_desc_prop = initial;
+				if(_obj.&_desc_prop && (_obj hasnt moved || _desc_prop == when_off)) { ! Note: when_closed in an alias of when_off
+					give _obj ~workflag;
+					new_line;
+					PrintOrRun(_obj, _desc_prop);
+					also_flag = true;
 				}
-			} else {
-				_desc_prop = initial;
 			}
-			if(_obj.&_desc_prop && (_obj hasnt moved || _desc_prop == when_off)) { ! Note: when_closed in an alias of when_off
-				give _obj ~workflag;
-				@new_line;
-				PrintOrRun(_obj, _desc_prop);
-				also_flag = true;
-			}
-		}
 
 		! write any remaining objects in a new paragraph
 		if(parent(_ceil) == 0) {
@@ -1679,24 +1859,19 @@ Global scope_cnt;
 			_you_can_see_1 = _ListObjsInOnMsg;
 			_you_can_see_2 = ".^";
 		}
-		newline_flag = true;
 		if(PrintContents(_you_can_see_1, _ceil, true)) print (string) _you_can_see_2;
 
 
 #IfDef OPTIONAL_PRINT_SCENERY_CONTENTS;
-		newline_flag = true;
-		objectloop(_obj in _ceil && _obj has scenery &&
-				(_obj has supporter ||
-					(_obj has container && _obj has transparent or open)) &&
-					child(_obj) ~= 0 &&
-					IndirectlyContains(_obj, player) == false) {
-			if(PrintContents(_ListObjsInOnMsg, _obj)) {
-				print (string) ". ";
-				newline_flag = false;
+		objectloop(_obj in _ceil)
+			if(_obj has scenery &&
+					(_obj has supporter ||
+						(_obj has container && _obj has transparent or open)) &&
+						child(_obj) ~= 0 &&
+						IndirectlyContains(_obj, player) == false) {
+				if(PrintContents(_ListObjsInOnMsg, _obj))
+					print ".^";
 			}
-		}
-		if(newline_flag == false)
-			print "^";
 #EndIf;
 
 		! Descend one level
@@ -1733,21 +1908,29 @@ Global scope_cnt;
 [ PANum p_m _n;
 	print "  ";
 	_n = p_m;
-	if(_n < 0)    { _n = -p_m; _n = _n*10; }
-	if(_n < 10)   { print "   "; jump _Panuml; }
-	if(_n < 100)  { print "  "; jump _Panuml; }
-	if(_n < 1000) { print " "; }
+	if(_n < 0)    _n = -10 * p_m;
+	if(_n < 10)   print " ";
+	if(_n < 100)  print " ";
+	if(_n < 1000) print " ";
 ._Panuml;
 	print p_m, " ";
 ];
 #EndIf;
 
-[ TryToTakeNoun _i _k _ancestor _after_recipient;
+[ TryToTakeNoun p_suppress_before_after _i _k _r _ancestor _after_recipient;
 	! Try to transfer the given item to the player. Return values:
 	! 	0: Success, no message has been printed
 	!   1: Failed, a message has been printed
 	!   2: Success, a message has been printed
 
+	_k = action;
+	if(p_suppress_before_after == false) {
+		action = ##Take;
+		_r = BeforeRoutines();
+		action = _k;
+		if (_r) rtrue;
+	}
+
 	! People cannot ordinarily be taken.
 	if(noun == player) { PrintMsg(MSG_TAKE_YOURSELF); rtrue; };
 #Ifdef DisallowTakeAnimate;
@@ -1777,10 +1960,12 @@ Global scope_cnt;
 #IfDef DEBUG;
 		if(debug_flag & 1) print "(", (name) _i, ").before()^";
 #EndIf;
-		_k = action; action = ##LetGo;
-		if (RunRoutines(_i, before)) { action = _k; rtrue; }
+		action = ##LetGo;
+		_r = RunRoutines(_i, before);
 		action = _k;
+		if (_r) rtrue;
 	}
+	
 
 	if(noun has scenery) { PrintMsg(MSG_TAKE_SCENERY); rtrue; }
 	if(noun has static) { PrintMsg(MSG_TAKE_STATIC); rtrue; }
@@ -1796,16 +1981,47 @@ Global scope_cnt;
 #IfDef DEBUG;
 		if(debug_flag & 1) print "(", (name) _after_recipient, ").after()^";
 #EndIf;
-		_k = action; action = ##LetGo;
+		action = ##LetGo;
 		_i = RunRoutines(_after_recipient, after);
 		action = _k;
 		if(_i) return 2;
 	}
 
+	if(p_suppress_before_after == false) {
+		action = ##Take;
+		_r = AfterRoutines();
+		action = _k;
+		if (_r) return 2;
+	}
+
 	rfalse;
 ];
 
-[ GoDir p_property _new_location _old_location _door_to _vehicle _vehicle_mode _saved_location;
+[ DoorTo p_door _door_to;
+	! Returns the location the door leads to (handling Simple Doors, as needed). 
+	! Value true means the exit is blocked and a message has been printed
+#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
+	_CheckDoorDirTo(p_door);
+#EndIf;
+	_door_to = p_door.door_to;
+#IfDef OPTIONAL_SIMPLE_DOORS;
+	if(_door_to == 0) {
+		! This is a Simple Door, where door_to has been left out
+		if(real_location == p_door.&found_in-->0)
+			_door_to = 1;
+		return p_door.&found_in-->_door_to;
+	} else {
+#EndIf;
+		! Normal Inform door
+		if(UnsignedCompare(_door_to, top_object) > 0)
+			return p_door.door_to();
+		return _door_to;
+#IfDef OPTIONAL_SIMPLE_DOORS;
+	}
+#EndIf;
+];
+
+[ GoDir p_property _new_location _old_location _vehicle _vehicle_mode _saved_location;
 	if(parent(player) ~= real_location) {
 		! special rule when in enterable (veichles)
 		! before routine for the object is called with Go dir, and returns
@@ -1845,39 +2061,9 @@ Global scope_cnt;
 			_new_location = 0;
 		else {
 			if(_new_location hasnt open) { PrintMsg(MSG_GO_DOOR_CLOSED, _new_location); rtrue; }
-#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
-#Ifdef OPTIONAL_SIMPLE_DOORS;
-			if(_new_location.&door_dir == 0 || (_new_location.&door_to == 0 && (_new_location.&found_in == 0 || _new_location.#found_in ~= 4))) {
-				_RunTimeError(ERR_OBJECT_HASNT_PROPERTY, _new_location);
+			_new_location = DoorTo(_new_location);
+			if(_new_location == 1)
 				rtrue;
-			}
-#Ifnot;
-			if(_new_location.&door_to == 0 || _new_location.&door_dir == 0) {
-				_RunTimeError(ERR_OBJECT_HASNT_PROPERTY, _new_location);
-				rtrue;
-			}
-#EndIf;
-#EndIf;
-			_door_to = _new_location.door_to;
-#IfDef OPTIONAL_SIMPLE_DOORS;
-			if(_door_to == 0) {
-				! This is a Simple Door, where door_to has been left out
-				if(real_location == _new_location.&found_in-->0)
-					_door_to = 1;
-				_new_location = _new_location.&found_in-->_door_to;
-			} else {
-#EndIf;
-				! Normal Inform door
-				if(UnsignedCompare(_door_to, top_object) > 0) {
-					_new_location = _new_location.door_to();
-	!				print "GoDir, door leads to ", (the) _new_location, "^";
-					if(_new_location == 1)
-						rtrue;
-				} else
-					_new_location = _door_to;
-#IfDef OPTIONAL_SIMPLE_DOORS;
-			}
-#EndIf;
 		}
 	}
 
@@ -1921,6 +2107,8 @@ Global scope_cnt;
 
 	if(AfterRoutines()) rtrue;
 	if(keep_silent) rtrue;
+	
+	PrintMsg(MSG_LOOK_BEFORE_ROOMNAME);
 	Look();
 ];
 
@@ -1933,33 +2121,55 @@ Global scope_cnt;
 
 #IfV3;
 	switch(p_v) {
-		'restart': print "restart"; return;
-		'restore': print "restore"; return;
-		'display': print "display"; return;
-		'present': print "present"; return;
-		'destroy': print "destroy"; return;
-		'torture': print "torture"; return;
-		'unscrew': print "unscrew"; return;
-#IfDef OPTIONAL_EXTENDED_VERBSET;
-		'embrace': print "embrace"; return;
-#EndIf;
-		'x//', 'examine': print "examine"; return;
+		'g//': print "again"; return;
 		'i//', 'inventory': print "inventory"; return;
 		'l//': print "look"; return;
-		'z//': print "wait"; return;
 		'q//': print "quit"; return;
-		'g//': print "again"; return;
-	}
-#IfNot;
-	switch(p_v) {
-		'x//': print "examine"; return;
-		'i//': print "inventory"; return;
-		'l//': print "look"; return;
+		'x//', 'examine': print "examine"; return;
 		'z//': print "wait"; return;
-		'q//': print "quit"; return;
-		'g//': print "again"; return;
 	}
-#EndIf;
 
 	print (address) p_v;
+
+	! For words longer than 6 characters, print the extra characters
+	switch(p_v) {
+		'destroy', 'display': print (char) 'y'; return;
+		'discard': print (char) 'd'; return;
+		'disrobe', 'restore', 'torture', 'verbose': print (char) 'e'; return;
+#Ifndef NO_PLACES;
+		'objects': print (char) 's'; return;
+#Endif;
+		'present', 'restart': print (char) 't'; return;
+		'superbrief': print "rief"; return;
+		'swallow', 'unscrew': print (char) 'w'; return;
+		'uncover': print (char) 'r'; return;
+		'version': print (char) 'n'; return;
+#IfDef OPTIONAL_EXTENDED_VERBSET;
+		'consult': print (char) 't'; return;
+		'embrace', 'squeeze': print (char) 'e'; return;
+		'purchase': print "se"; return;
+		'transfer': print "er"; return;
+#EndIf;
+#IfDef OPTIONAL_EXTENDED_METAVERBS;
+		'noscript', 'unscript': print "pt"; return;
+		'transcript': print "ript"; return;
+#EndIf;
+	}
+#IfNot; ! This is z5+
+	switch(p_v) {
+		'superbrief': print "superbrief"; return;
+		'g//': print "again"; return;
+		'i//': print (address) 'inventory'; return;
+		'l//': print "look"; return;
+		'q//': print "quit"; return;
+		'x//': print (address) 'examine'; return;
+		'z//': print "wait"; return;
+#IfDef OPTIONAL_EXTENDED_METAVERBS;
+		'transcript': print "transcript"; return;
+#EndIf;
+	}
+	print (address) p_v;
+#EndIf;
+
 ];
+
