@@ -244,7 +244,6 @@ Default MSG_LOCK_KEY_DOESNT_FIT 74;
 Default MSG_LOCK_DEFAULT 75;
 Default MSG_DISROBE_NOT_WEARING 76;
 Default MSG_DISROBE_DEFAULT 77;
-Default MSG_REMOVE_CLOSED 78;
 Default MSG_REMOVE_NOT_HERE 79;
 Default MSG_SEARCH_IN_IT_ISARE 80;
 Default MSG_SEARCH_ON_IT_ISARE 81;
@@ -525,14 +524,12 @@ Constant SKIP_MSG_ASKFOR_DEFAULT;
 #Iffalse MSG_INSERT_NOT_OPEN < 1000;
 #Iffalse MSG_GO_DOOR_CLOSED < 1000;
 #Iffalse MSG_EMPTY_IS_CLOSED < 1000;
-#Iffalse MSG_REMOVE_CLOSED < 1000;
 Constant SKIP_MSG_ENTER_NOT_OPEN;
 #Endif;
 #Endif;
 #Endif;
 #Endif;
 #Endif;
-#Endif;
 
 #Iffalse MSG_GIVE_PLAYER < 1000;
 #Iffalse MSG_TAKE_ALREADY_HAVE < 1000;
@@ -797,18 +794,18 @@ Constant SKIP_MSG_EXAMINE_DARK;
 #Endif;
 #Ifndef SKIP_MSG_GIVE_DEFAULT;
 	MSG_GIVE_DEFAULT, MSG_SHOW_DEFAULT:
-		print_ret (The) second, " doesn't seem interested.";
+		print_ret (The) second, " ", (DoOrDoes) second, "n't seem interested.";
 #Endif;
 #Ifndef SKIP_MSG_ASKFOR_DEFAULT;
 	MSG_ASKFOR_DEFAULT, MSG_ASKTO_DEFAULT, MSG_ORDERS_WONT:
 	! p_arg_1 = the actor which the player has asked to do something.
-		print_ret (The) p_arg_1, " has better things to do.";
+		print_ret (The) p_arg_1, " ", (HaveOrHas) p_arg_1, " better things to do.";
 #Endif;
 #Ifndef SKIP_MSG_ENTER_NOT_OPEN;
 	MSG_ENTER_NOT_OPEN, MSG_EXIT_NOT_OPEN, MSG_INSERT_NOT_OPEN,
-	MSG_GO_DOOR_CLOSED, MSG_EMPTY_IS_CLOSED, MSG_REMOVE_CLOSED:
+	MSG_GO_DOOR_CLOSED, MSG_EMPTY_IS_CLOSED:
 	! p_arg_1 = the object which is closed, thus blocking the player's action.
-		"You can't, since ", (the) p_arg_1, " ", (isorare) p_arg_1, " closed.";
+		"You can't, since ", (ObjIs) p_arg_1, " closed.";
 #Endif;
 #Ifndef SKIP_MSG_GIVE_PLAYER;
 	MSG_GIVE_PLAYER, MSG_TAKE_ALREADY_HAVE:
@@ -923,7 +920,11 @@ Constant SKIP_MSG_EXAMINE_DARK;
 #EndIf;
 #IfTrue MSG_PARSER_DONT_UNDERSTAND_WORD < 1000;
 	MSG_PARSER_DONT_UNDERSTAND_WORD:
-		print "Sorry, I don't understand what ~";
+		print "Sorry, I don't understand ";
+		if(oops_unfixed) {
+			"that word either.";
+		}
+		print "what ~";
 		_PrintUnknownWord();
 		"~ means.";
 #EndIf;
@@ -969,7 +970,7 @@ Constant SKIP_MSG_EXAMINE_DARK;
 #EndIf;
 #Ifndef SKIP_MSG_LOCK_KEY_DOESNT_FIT;
 	MSG_LOCK_KEY_DOESNT_FIT, MSG_UNLOCK_KEY_DOESNT_FIT:
-		print_ret (The) second, " doesn't seem to fit the lock.";
+		print_ret (The) second, " ", (DoOrDoes) second, "n't seem to fit the lock.";
 #Endif;
 #IfTrue MSG_EXAMINE_CLOSED < 1000;
 	MSG_EXAMINE_CLOSED:
@@ -978,7 +979,7 @@ Constant SKIP_MSG_EXAMINE_DARK;
 #Endif;
 #IfTrue MSG_REMOVE_NOT_HERE < 1000;
 	MSG_REMOVE_NOT_HERE:
-		"But ", (the) noun, " isn't there now.";
+		"But ", (ObjIs) noun, "n't there now.";
 #EndIf;
 #IfTrue MSG_SEARCH_IN_IT_ISARE < 1000;
 	MSG_SEARCH_IN_IT_ISARE:
@@ -1002,7 +1003,7 @@ Constant SKIP_MSG_EXAMINE_DARK;
 #EndIf;
 #IfTrue MSG_SEARCH_CANT_SEE_CLOSED < 1000;
 	MSG_SEARCH_CANT_SEE_CLOSED:
-		"You can't see inside, since ", (the) noun, " ", (IsorAre) noun, " closed.";
+		"You can't see inside, since ", (ObjIs) noun, " closed.";
 #EndIf;
 #IfTrue MSG_EXAMINE_ONOFF < 1000;
 	MSG_EXAMINE_ONOFF:
@@ -1223,6 +1224,19 @@ default:
 	print (The) p_obj, " ", (isorare) p_obj;
 ];
 
+[ ObjIs p_obj;
+	print (the) p_obj, " ", (isorare) p_obj;
+];
+
+[ DoOrDoes p_obj;
+	print "do";
+	if (p_obj hasnt pluralname) print "es";
+];
+
+[ HaveOrHas p_obj;
+	if (p_obj has pluralname) print "have"; else print "has";
+];
+
 [ IsorAre p_obj;
 	if (p_obj has pluralname || p_obj == player) print "are"; else print "is";
 ];
