@@ -54,31 +54,6 @@ Include "grammar.h";
 ];
 #Endif;
 
-[ IndirectlyContains p_o1 p_o2;
-	! Does o1 indirectly contain o2?  (Same as testing if o1 is one of the ancestors of o2.)
-	@jz p_o2 ?rfalse;
-.recheck;
-	if (p_o1 == p_o2) rtrue;
-	p_o2 = parent(p_o2);
-	@jz p_o2 ?~recheck;
-	rfalse;
-];
-
-[ CommonAncestor p_o1 p_o2 _i _j;
-	! Find the nearest object indirectly containing o1 and o2,
-	! or return 0 if there is no common ancestor.
-	_i = p_o1;
-	while (_i) {
-		_j = p_o2;
-		while (_j) {
-			if (_j == _i) return _i;
-			_j = parent(_j);
-		}
-		_i = parent(_i);
-	}
-	return 0;
-];
-
 #Ifv3;
 [ ChangeFgColour p_colour;
 	p_colour = 1; ! Avoid warning
@@ -87,6 +62,12 @@ Include "grammar.h";
 
 #IfV5;
 
+[ OzmooColoursAvailable;
+	if($3a-->0 == $4f5a)
+		rtrue;
+	rfalse;
+];
+
 [ ClearScreen window;
 	if (clr_on) {
 		@set_colour clr_fg clr_bg;
@@ -177,16 +158,22 @@ Constant ONE_SPACE_STRING = " ";
 
 [ _PrintStatusLineTime _h _pm;
 	if (screen_width > 29) {
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+		parser_one = 1;
+#Endif;
 		if (screen_width > 39) {
 			if (screen_width > 66) {
 				! Width is 67-, print "Time: 12:34 pm" with some space to the right
 				_PrintSpacesOrMoveBack(20, TIME__TX);
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+				parser_one = 6;
+#Endif;
 			} else {
-				! Width is 40-66, print "Time: 12:34 pm" at right edge
+				! Width is 40-66, print "Time: 12:34 pm" one character fromm right edge
 				_PrintSpacesOrMoveBack(15, TIME__TX);
 			}
 		} else {
-			! Width is 30-, print "12:34 pm" at right edge
+			! Width is 30-, print "12:34 pm" one character from right edge
 			_PrintSpacesOrMoveBack(9, ONE_SPACE_STRING);
 		}
 		_h = status_field_1;
@@ -208,6 +195,24 @@ Constant ONE_SPACE_STRING = " ";
 	}
 ];
 
+[ _NumberLength p_number _length;
+	_length = 1;
+	if(p_number < 0) { _length++; p_number = -p_number; }
+	if(p_number > 9) {
+		_length++;
+		if(p_number > 99) {
+			_length++;
+			if(p_number > 999) {
+				_length++;
+				if(p_number > 9999) {
+					_length++;
+				}
+			}
+		}
+	}
+	return _length;
+];
+
 [ _PrintStatusLineScore;
 #Ifdef OPTIONAL_SL_NO_SCORE;
 #Ifndef OPTIONAL_SL_NO_MOVES;
@@ -216,16 +221,28 @@ Constant ONE_SPACE_STRING = " ";
 		if (screen_width < 30) {
 			! Width is 25-29, only print moves as "0"
 			_PrintSpacesOrMoveBack(4, ONE_SPACE_STRING);
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+			parser_one = 4;
+#Endif;
 		} else {
 			! Width is 30-, print "Moves: 0"
 			if (screen_width > 52) {
 				! Width is 53+, leave some space to the right
 				_PrintSpacesOrMoveBack(15, MOVES__TX);
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+				parser_one = 8;
+#Endif;
 			} else {
 				_PrintSpacesOrMoveBack(11, MOVES__TX);
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+				parser_one = 4;
+#Endif;
 			}
 		}
 		print status_field_2;
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+		parser_one = parser_one - _NumberLength(status_field_2);
+#Endif;
 	}
 #Endif; ! Ifndef NO_MOVES
 #Ifnot;
@@ -235,22 +252,44 @@ Constant ONE_SPACE_STRING = " ";
 			! Width is 25-29, only print score as "0", no moves
 			_PrintSpacesOrMoveBack(3, ONE_SPACE_STRING);
 			print status_field_1;
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+			parser_one = 3 - _NumberLength(status_field_1);
+#Endif;
 		} else {
 #Ifdef OPTIONAL_SL_NO_MOVES;
 	! Show score only
 			! Width is 30-, print "Score: 0"
-			if(screen_width < 55)
+			if(screen_width < 55) {
 				_PrintSpacesOrMoveBack(10, SCORE__TX);
-			else
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+				parser_one = 3;
+#Endif;
+			} else {
 				_PrintSpacesOrMoveBack(13, SCORE__TX);
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+				parser_one = 6;
+#Endif;
+			}
 			print status_field_1;
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+			parser_one = parser_one - _NumberLength(status_field_1);
+#Endif;
 #Ifnot;
 	! Show score + moves
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+			parser_one = 7 - _NumberLength(status_field_1);
+#Endif;
 			if (screen_width > 66) {
 				! Width is 67-, print "Score: 0 Moves: 0"
 				_PrintSpacesOrMoveBack(28, SCORE__TX);
 				print status_field_1;
+#Ifdef OPTIONAL_NON_FLASHING_STATUSLINE;
 				_PrintSpacesOrMoveBack(14, MOVES__TX);
+#Ifnot;
+				FastSpaces(parser_one - 1);
+				print (string) MOVES__TX;
+				parser_one = 7;
+#Endif;
 			} else {
 				if (screen_width > 36) {
 					! Width is 37-66, print "Score: 0/0"
@@ -263,13 +302,16 @@ Constant ONE_SPACE_STRING = " ";
 				@print_char '/';
 			}
 			print status_field_2;
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+			parser_one = parser_one - _NumberLength(status_field_2);
+#Endif;
 #Endif;
 		}
 	}
 #Endif;
 ];
 
-[ DrawStatusLine _visibility_ceiling;
+[ DrawStatusLine p_linefeeds _visibility_ceiling;
 	! For wide screens (67+ columns):
 	! * print a space before room name, and "Score: xxx  Moves: xxxx" to the right.
 	! * Room names up to 39 characters are never truncated.
@@ -278,12 +320,17 @@ Constant ONE_SPACE_STRING = " ";
 	! * Print "Score: xxx/yyyy", "xxx/yyyy", "xxx" or nothing, depending on screen width
 	! * Room names up to 21 characters are never truncated. On a 40 column screen, room names up to 24 characters are never truncated.
 
+	! If called with p_linefeeds == true, print the number of linefeeds needed
+	! to make sure text at start of game doesn't get covered by statusline
+	if(p_linefeeds) { new_line; return; }
+
 	! If there is no player location, we shouldn't try to draw status window
 	if (location == nothing || parent(player) == nothing)
 		return;
 
 	_StatusLineHeight(statusline_height);
 	_MoveCursor(1, 1); ! This also sets the upper window as active.
+	parser_one = 1000;
 #Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
 	FastSpaces(screen_width);
 	_MoveCursor(1, 1);
@@ -318,9 +365,12 @@ Constant ONE_SPACE_STRING = " ";
 	#Endif;
 #Endif;
 
-	! Regardless of what kind of status line we have printed, print spaces to the end.
 #Ifdef OPTIONAL_NON_FLASHING_STATUSLINE;
+	! Regardless of what kind of status line we have printed, print spaces to the end.
 	_PrintSpacesOrMoveBack(-1);
+#Ifnot;
+	if(parser_one > 0 && parser_one < screen_width) 
+		FastSpaces(parser_one);
 #Endif;
 	_MainWindow(); ! set_window
 ];
@@ -369,15 +419,55 @@ Constant ONE_SPACE_STRING = " ";
 ];
 #EndIf;
 
-#IfDef DEBUG;
+!#IfDef DEBUG;
 #IfnDef DebugParseNameObject;
 [ DebugParseNameObject p_obj;
 	@inc p_obj; ! Just to get rid of warning that p_obj isn't used
 	rfalse;
 ];
 #Endif;
+!#Endif;
+
+#Ifdef IsARoom;
+[ _RoomLike p_obj _verdict _return_code;
+#Ifnot;
+[ _RoomLike p_obj _verdict;
 #Endif;
 
+	! Return true if p_obj seems to be a room
+	if(p_obj > Directions && p_obj <= top_object && p_obj in nothing
+			&& p_obj provides description
+			&& (~~(p_obj provides describe or life or found_in))
+			&& (~~DebugParseNameObject(p_obj))) {
+		if(p_obj has edible or talkable or supporter or container or transparent
+				or concealed or scenery or static or animate or clothing
+				or pluralname or switchable or door or lockable)
+			jump _decided;
+#Ifndef OPTIONAL_NO_DARKNESS;
+		if(p_obj == thedark) jump _decided;
+#Endif;
+		_verdict = true;
+	}
+._decided;
+#Ifdef IsARoom;
+	_return_code = IsARoom(p_obj, _verdict);
+	if(_return_code)
+		_verdict = 2 - _return_code;
+#Endif;
+	return _verdict;
+];
+
+[ _ObjectLike p_obj;
+	if(p_obj > Directions && p_obj <= top_object && p_obj notin Class && _RoomLike(p_obj) == false) {
+#Ifndef OPTIONAL_NO_DARKNESS;
+		if(p_obj == thedark) rfalse;
+#Endif;	
+		rtrue;
+	}
+	rfalse;
+];
+
+
 #Ifdef OPTIONAL_LANGUAGE_NUMBER;
 
 [ LanguageNumber n f;
@@ -426,6 +516,10 @@ Constant ONE_SPACE_STRING = " ";
 
 
 [ PrintShortName o;
+	if(caps_mode == 2) 
+		caps_mode = true;
+	else 
+		caps_mode = false;
 	if (o == 0) { print "nothing"; rtrue; }
 	switch (metaclass(o)) {
 	  Routine:  print "<routine ", o, ">"; rtrue;
@@ -437,6 +531,9 @@ Constant ONE_SPACE_STRING = " ";
 ];
 
 [ _PrintObjName p_obj p_form;
+	caps_mode = false;
+	if(p_form == FORM_CDEF)
+		caps_mode = 2;
 	if(p_obj hasnt proper) {
 		if(p_form == FORM_CDEF) {
 			print "The ";
@@ -461,39 +558,53 @@ Constant ONE_SPACE_STRING = " ";
 	return IS_STR;
 ];
 
-[ _PrintAfterEntry p_obj;
+[ _PrintAfterEntry p_obj _contents _newline _started;
+	_newline = c_style & NEWLINE_BIT;
 #Ifndef OPTIONAL_NO_DARKNESS;
 	if(p_obj has light && p_obj hasnt animate) print " (providing light)";
 #Endif;
 	if(p_obj has worn && action == ##Inv) print " (worn)";
-	if(p_obj has container && p_obj hasnt open) print " (which is closed)";
-	if(p_obj has container && (p_obj has open || p_obj has transparent)) {
-		if(PrintContentsFromR(1, child(p_obj)) == 0) {
-			print " (which is empty)";
-			if(c_style & NEWLINE_BIT)
-				new_line;
+	if(p_obj has container) {
+		_contents = PrintContentsFromR(1, child(p_obj));
+		if(p_obj hasnt open && p_obj hasnt transparent) {
+			print " (which ",(IsorAre) p_obj," closed)";
+			if(_newline) new_line;
 		} else {
-			if(c_style & NEWLINE_BIT == 0)
-				print " (which contains ";
-			else {
-				if(p_obj has open)
-					print " (which is open)";
-				new_line;
+			if(p_obj has openable && (p_obj has transparent || _contents == 0)) {
+				print " (which ", (IsorAre) p_obj, " ";
+				if(p_obj has open) { print "open"; _started = 1; }
+				else { print "closed"; _started = 2; }
 			}
-			c_style = c_style & ~ISARE_BIT;
-			PrintContentsFromR(0, child(p_obj));
-			if(c_style & NEWLINE_BIT == 0) {
-				print (char) ')';
+			if(_contents == 0) {
+				print (char) ' ';
+				switch(_started) {
+				0: print "(which ", (IsorAre) p_obj;
+				1: print "but";
+				2: print "and";
+				}
+				print " empty)";
+				if(_newline) new_line;
+			} else {
+				if(_newline == 0) {
+					if(_started) print " and";
+					else print " (which";
+					print " contain", (SingularS) p_obj, " ";
+				} else {
+					if(_started) print (char) ')';
+					new_line;
+				}
+				c_style = c_style & ~ISARE_BIT;
+				PrintContentsFromR(0, child(p_obj));
+				if(_newline == 0) print (char) ')';
 			}
 		}
-
 	} else if(p_obj has supporter) {
-		if(c_style & NEWLINE_BIT) {
+		if(_newline) {
 			new_line;
 			PrintContentsFromR(0, child(p_obj));
 		} else
 			if(PrintContents(" (on which ", p_obj, ISARE_BIT)) print (char) ')';
-	} else if(c_style & NEWLINE_BIT)
+	} else if(_newline)
 		new_line;
 ];
 
@@ -798,7 +909,7 @@ Constant ONE_SPACE_STRING = " ";
 
 [ _PrintContentsShowObj p_obj;
 	! Return true if object should be shown in list, false if not
-	if(p_obj ~= parent(player) && ! don't print container when player in it
+	if(IndirectlyContains(p_obj, player) == false && ! don't print container when player in it 
 			(pc_depth > 0 || c_style & WORKFLAG_BIT == 0 || p_obj has workflag) &&
 	! Hide concealed and scenery unless taking inventory
 			(action == ##Inv || (p_obj hasnt concealed && p_obj hasnt scenery)))
@@ -863,28 +974,63 @@ Constant ONE_SPACE_STRING = " ";
 ];
 
 [ PrintOrRun p_obj p_prop p_no_string_newline _val;
-	_val = p_obj.p_prop;
-	if (p_obj.#p_prop > WORDSIZE || _val ofclass Routine) return RunRoutines(p_obj, p_prop);
+	if (p_obj.#p_prop > WORDSIZE || (_val = p_obj.p_prop) ofclass Routine) return RunRoutines(p_obj, p_prop);
 	if(_val ofclass String) {
-		print (string) p_obj.p_prop;
+		print (string) _val;
 		if(p_no_string_newline == 0) new_line;
 	}
 ];
 
+[ CommonAncestor p_o1 p_o2 _i _j;
+	! Find the nearest object indirectly containing o1 and o2,
+	! or return 0 if there is no common ancestor.
+	_i = p_o1;
+	while (_i) {
+		_j = p_o2;
+		while (_j) {
+			if (_j == _i) return _i;
+			_j = parent(_j);
+		}
+		_i = parent(_i);
+	}
+	return 0;
+];
+
+[ IndirectlyContains p_o1 p_o2;
+	! Does o1 indirectly contain o2?  (Same as testing if o1 is one of the ancestors of o2.)
+	@jz p_o2 ?rfalse;
+._recheck;
+	@je p_o1 p_o2 ?rtrue;
+!	if (p_o1 == p_o2) rtrue;
+	p_o2 = parent(p_o2);
+	@jz p_o2 ?~_recheck;
+	rfalse;
+];
+
 [ MoveFloatingObjects _i _j _o _len _obj;
-	while((_obj = floating_objects-->_i) ~= 0) {
-		if(IndirectlyContains(player, _obj))
-			jump _continue_loop;
-		_len = _obj.#found_in;
+	_i--;
+!	while((_obj = floating_objects-->_i) ~= 0) {
+._next_floating;
+		_i++;
+		_obj = floating_objects-->_i;
+		if(_obj == 0)
+			jump _done_floating;
+		if(parent(_obj) ~= 0 && IndirectlyContains(player, _obj))
+			jump _next_floating;
 		if(_obj has absent)
 			jump _isnt_present;
-		else if(_len == 2 && UnsignedCompare(_obj.found_in, top_object) > 0) {
+		_len = _obj.#found_in;
+		if(_len == 2 && UnsignedCompare(_obj.found_in, top_object) > 0) {
 			if(RunRoutines(_obj, found_in))
 				jump _is_present;
 			jump _isnt_present;
 		} else {
 			_j = _obj.&found_in;
+#Ifv5;
+			@log_shift _len (-1) -> _len; ! Divide by 2
+#Ifnot;
 			_len = _len / 2;
+#Endif;
 			_len = _len - 1;
 ._check_next_value;
 				_o = _j-->_len;
@@ -896,18 +1042,19 @@ Constant ONE_SPACE_STRING = " ";
 			@dec_chk _len 0 ?~_check_next_value;
 ._isnt_present;
 			remove _obj;
-			jump _continue_loop;
+			jump _next_floating;
 ._is_present;
 			if(_obj notin location)
 				move _obj to location;
 		}
-._continue_loop;
-		_i++;
-	}
+		jump _next_floating;
+!._continue_loop;
+!		_i++;
+!	}
+._done_floating;
 	! It's not certain that scope has been modified, but PlayerTo relies on it
 	! being set.
 	scope_modified = true;
-!	print "MFO done!^";
 ];
 
 [ CalculateVisibilityCeiling;
@@ -932,7 +1079,7 @@ Constant ONE_SPACE_STRING = " ";
 	location = real_location;
 	MoveFloatingObjects(); ! Also sets scope_modified = true;
 #Ifndef OPTIONAL_NO_DARKNESS;
-	_UpdateDarkness();
+	_UpdateDarkness(true);
 #Endif;
 ._recheck_visibility_ceil;
 	_vc = CalculateVisibilityCeiling();
@@ -966,8 +1113,8 @@ Constant ONE_SPACE_STRING = " ";
 	_old_lookmode = lookmode;
 	if(p_flag==false)
 		lookmode = 2;
-	if(p_flag==false or 2 && deadflag == GS_PLAYING)
-		<Look>;
+	if(p_flag~=true && deadflag == GS_PLAYING)
+		Look();
 	lookmode = _old_lookmode;
 ];
 
@@ -981,17 +1128,20 @@ Constant ONE_SPACE_STRING = " ";
 ];
 
 #Ifndef OPTIONAL_NO_DARKNESS;
-[ _UpdateDarkness p_look _ceil _old_darkness _darkness;
+[ _UpdateDarkness p_silent _ceil _old_darkness _darkness;
 	if(location == thedark) _old_darkness = true;
 	_ceil = ScopeCeiling(player);
 	if(_LookForLightInObj(_ceil, _ceil) == false) _darkness = true;
 	if(_darkness ~= _old_darkness) scope_modified = true;
 	if(_darkness) {
+		if(_old_darkness == false && p_silent == false) PrintMsg(MSG_NOW_DARK);
 		location = thedark;
 	} else {
 		location = real_location;
-		if(_old_darkness == true && p_look == true)
+		if(_old_darkness && p_silent == false) {
+			new_line;
 			<Look>;
+		}
 	}
 ];
 
@@ -1015,6 +1165,12 @@ Include "parser.h";
 #IfDef DEBUG;
 	if(debug_flag & 2) TraceAction();
 #EndIf;
+	if(input_action == -2) {
+		input_action = action;
+		input_noun = noun;
+		input_second = second;
+		input_direction = selected_direction;
+	}
 	if ((meta || (BeforeRoutines() == false)) && action < 4096) {
 		@push run_after_routines_msg; @push run_after_routines_arg_1;
 		run_after_routines_msg = 0;
@@ -1026,7 +1182,7 @@ Include "parser.h";
 		! should be run. If it has set it to another value, this message should
 		! be printed unless after routines returns true.
 		if(run_after_routines_msg && AfterRoutines() == false &&
-				run_after_routines_msg ~= 0 or 1 &&
+				run_after_routines_msg ~= 1 &&
 				keep_silent == false)
 			PrintMsg(run_after_routines_msg, run_after_routines_arg_1);
 		@pull run_after_routines_arg_1; @pull run_after_routines_msg;
@@ -1072,7 +1228,7 @@ Include "parser.h";
 #EndIf;
 #IfDef DEBUG;
 #IfV3;
-		if(debug_flag & 1) print "(", (name) _obj, ").",(property) p_property,"()^";
+		if(debug_flag & 1) print "[ ~", (name) _obj, "~.",(property) p_property,"() ]^";
 #EndIf;
 #EndIf;
 		if(RunRoutines(_obj, p_property) && p_break) {
@@ -1091,7 +1247,7 @@ Include "parser.h";
 	if(location has reactive && location.&each_turn ~= 0) {
 #IfDef DEBUG;
 #IfV3;
-		if(debug_flag & 1) print "(", (name) location, ").each_turn()^";
+		if(debug_flag & 1) print "[ ~", (name) location, "~.each_turn() ]^";
 #EndIf;
 #EndIf;
 		RunRoutines(location, each_turn);
@@ -1114,21 +1270,20 @@ Include "parser.h";
 ];
 
 [ BeforeRoutines;
+	if(real_location == 0) rfalse;
 
 	GetScopeCopy(player, REACT_BEFORE_REASON); ! later used by _RunReact
 
 #IfDef GamePreRoutine;
 #IfDef DEBUG;
-#IfV3;
-	if(debug_flag & 1) print "GamePreRoutine()^";
-#EndIf;
+	if(debug_flag & 1) print "[ GamePreRoutine() ]^";
 #EndIf;
 	if(RunEntryPointRoutine(GamePreRoutine)) rtrue;
 #EndIf;
 
 #IfDef DEBUG;
 #IfV3;
-	if(debug_flag & 1) print "player.orders()^";
+	if(debug_flag & 1) print "[ player.orders() ]^";
 #EndIf;
 #EndIf;
 	if(RunRoutines(player, orders)) rtrue;
@@ -1152,16 +1307,16 @@ Include "parser.h";
 
 #IfDef DEBUG;
 #IfV3;
-	if(debug_flag & 1) print "(", (name) real_location, ").before()^";
-#EndIf;
+	if(debug_flag & 1) print "[ ~", (name) real_location, "~.before() ]^";
 #EndIf;
+#EndIf;	
 	if(real_location.&before) {
 		if(RunRoutines(real_location, before)) rtrue;
 	}
 	if(inp1 > 1) {
 #IfDef DEBUG;
 #IfV3;
-		if(debug_flag & 1) print "(", (name) inp1, ").before()^";
+		if(debug_flag & 1) print "[ ~", (name) inp1, "~.before() ]^";
 #EndIf;
 #EndIf;
 		if(inp1.&before) {
@@ -1174,6 +1329,7 @@ Include "parser.h";
 [ AfterRoutines;
 	! react_after - Loops over the scope to find possible react_before routines
 	! to run in each object, if it's found stop the action by returning true
+	if(real_location == 0) rfalse;
 
 	GetScopeCopy(player, REACT_AFTER_REASON); ! later used by _RunReact
 
@@ -1196,7 +1352,7 @@ Include "parser.h";
 
 #IfDef DEBUG;
 #IfV3;
-	if(debug_flag & 1) print "(", (name) real_location, ").after()^";
+	if(debug_flag & 1) print "[ ~", (name) real_location, "~.after() ]^";
 #EndIf;
 #EndIf;
 	if(real_location.&after) {
@@ -1205,7 +1361,7 @@ Include "parser.h";
 	if(inp1 > 1) {
 #IfDef DEBUG;
 #IfV3;
-		if(debug_flag & 1) print "(", (name) inp1, ").after()^";
+		if(debug_flag & 1) print "[ ~", (name) inp1, "~.after() ]^";
 #EndIf;
 #EndIf;
 		if(inp1.&after) {
@@ -1214,9 +1370,7 @@ Include "parser.h";
 	}
 #IfDef GamePostRoutine;
 #IfDef DEBUG;
-#IfV3;
-	if(debug_flag & 1) print "GamePostRoutine()^";
-#EndIf;
+	if(debug_flag & 1) print "[ GamePostRoutine() ]^";
 #EndIf;
 	if(RunEntryPointRoutine(GamePostRoutine)) rtrue;
 #EndIf;
@@ -1226,7 +1380,7 @@ Include "parser.h";
 [ RunLife p_actor p_reason;
 #IfDef DEBUG;
 #IfV3;
-	if(debug_flag & 1 && p_actor provides life) print "(", (name) p_actor, ").life()^";
+	if(debug_flag & 1 && p_actor provides life) print "[ ~", (name) p_actor, "~.life() ]^";
 #EndIf;
 #EndIf;
 	return RunRoutines(p_actor, life, p_reason);
@@ -2048,17 +2202,21 @@ Include "parser.h";
 		switch (n&$C0) { 0: n=1; $40: n=2; $80: n=n&$3F; }
 	}
 ! print "CA_Pr(3) obj = ", obj,", id = ", id,", a = ", a, "^";
-	for (:2*m<n:m++) {
-!   print "Considering routine at ", x+2*m,": ", x-->m, "^";
-		if (x-->m==$ffff) rfalse;
-		switch(Z__Region(x-->m)) {
+	for (:m+m<n:m++) {
+		z = x-->m;
+!   print "Considering routine at ", x+2*m,": ", z, "^";
+		if (z==$ffff) rfalse;
+		switch(Z__Region(z)) {
 		2:
 			s = sender; sender = self; self = obj; s2 = sw__var;
 !	   switch(y) {
 !	   0:
 !		 z = indirect(x-->m);
 !	   1:
-			z = indirect(x-->m, a);
+			if(a)
+				z = indirect(z, a);
+			else
+				z = indirect(z);
 !	   2:
 !		 z = indirect(x-->m, a, b);
 !	   3:
@@ -2073,9 +2231,9 @@ Include "parser.h";
 			self = sender; sender = s; sw__var = s2;
 			if (z ~= 0) return z;
 		3:
-			print_ret (string) x-->m;
+			print_ret (string) z;
 		default:
-			return x-->m;
+			return z;
 		}
 	}
 	rfalse;
@@ -2090,7 +2248,12 @@ Include "parser.h";
 Object selfobj "you"
 	with
 		name 'me' 'myself' 'self',
-		short_name  "yourself",
+!		short_name  "yourself",
+		short_name  [; 
+			if(caps_mode) { print "You"; rtrue; }
+			print "yourself";
+			rtrue;
+		],
 		description "As good-looking as ever.",
 		before NULL,
 		after NULL,
@@ -2163,7 +2326,7 @@ Object thedark "Darkness"
 	RunEntryPointRoutine(TimePasses);
 #Endif;
 #Ifndef OPTIONAL_NO_DARKNESS;
-	_UpdateDarkness(true);
+	_UpdateDarkness();
 #Endif;
 
 	if(update_moved || child(player) ~= last_player_child or 0) {
@@ -2231,6 +2394,8 @@ Object thedark "Darkness"
 	_i = Directions;
 .objloop;
 #Ifndef OPTIONAL_MANUAL_REACTIVE;
+		@get_next_prop _i 0 -> _v;
+		@jl _v react_before ?back_to_objloop;
 		@get_prop_addr _i react_before -> _v;
 		@jz _v ?~is_reactive;
 		@get_prop_addr _i react_after -> _v;
@@ -2281,9 +2446,13 @@ Object thedark "Darkness"
 	screen_width = HDR_SCREENWCHARS->0;
 #EndIf;
 	dict_start = HDR_DICTIONARY-->0;
-	dict_entry_size = dict_start->(dict_start->0 + 1);
-	dict_start = dict_start + dict_start->0 + 4;
+	_i = dict_start->0;
+	dict_entry_size = dict_start->(_i + 1);
+	dict_start = dict_start + _i + 4;
 	dict_end = dict_start + (dict_start - 2)-->0 * dict_entry_size;
+#Ifdef OPTIONAL_EXTENDED_METAVERBS;
+	transcript_mode = (HDR_GAMEFLAGS-->0) & 1;
+#Endif;
 
 	parse->0 = MAX_INPUT_WORDS;
 #IfV5;
@@ -2310,18 +2479,15 @@ Object thedark "Darkness"
 	places_score = 0;
 	things_score = 0;
 #EndIf;
-#IfDef TASKS_PROVIDED;
-	#Ifv5;
-		@copy_table task_done 0 NUMBER_TASKS;
-	#Ifnot;
-		for(_i = 0 : _i < NUMBER_TASKS : _i++) task_done->_i = 0;
-	#EndIf;
-#EndIf;
+	! We used to set all elements of task_done to 0 here, but the compiler 
+	! already does that when creating the array.
 #EndIf;
 
 #IfV5;
-	new_line; ! So the first line of text isn't covered by the statusline
+	DrawStatusLine(true); ! So the first line of text isn't covered by the statusline
 #Endif;
+	_InitObjects(); ! Give reactive attribute as needed + list floating objects
+
 	_j = Initialise();
 
 	objectloop (_i in player) give _i moved ~concealed;
@@ -2332,7 +2498,6 @@ Object thedark "Darkness"
 		@new_line;
 	}
 
-	_InitObjects(); ! after initialise since location set there
 	if(parent(player) == 0) { _i = location; location = 0; PlayerTo(_i); }
 
 	@new_line;
@@ -2357,6 +2522,19 @@ Object thedark "Darkness"
 #Endif;
 		if(parse->1 == 0) {
 			_ReadPlayerInput();
+#ifv5;
+			if(buffer->1 ~= 0 && buffer->2 == 42) { ! asterisk
+#ifnot;
+			if(buffer->1 == 42) { ! asterisk
+#endif;
+#ifdef OPTIONAL_EXTENDED_METAVERBS;
+				if(transcript_mode) PrintMsg(MSG_COMMENT_TRANSCRIPT);
+				else PrintMsg(MSG_COMMENT_NO_TRANSCRIPT);
+#Ifnot;
+				PrintMsg(MSG_COMMENT_NO_TRANSCRIPT);
+#Endif;
+				jump _abort_current_input;
+			}
 			_disallow_complex_again = false;
 #Ifdef OPTIONAL_PROVIDE_UNDO_FINAL;
 			if(parse-->1 == 'undo') {
@@ -2388,11 +2566,13 @@ Object thedark "Darkness"
 		timer1 = 0-->2;
 #Endif;
 		if(action == ##OopsCorrection) {
-			if(_again_saved && _parser_oops > 0) {
+			if(_again_saved && _parser_oops ~= 0) {
 				_CopyInputArray(buffer2, buffer);
 				_CopyParseArray(parse2, parse);
 				num_words = parse -> 1;
 				_parser_oops-->0 = special_word;
+				if(special_word == 0)
+					oops_unfixed = true;
 				_again_saved = false;
 				jump _try_same_command_again;
 			} else {
@@ -2435,7 +2615,7 @@ Object thedark "Darkness"
 		}
 
 #Ifdef AfterLife;
-		if(deadflag ~= GS_PLAYING or GS_WIN) {
+		if(deadflag ~= GS_PLAYING or GS_WIN or GS_QUIT) {
 			! we died somehow, use entry routine to give
 			! a chance of resurrection
 			RunEntryPointRoutine(AfterLife);
@@ -2453,10 +2633,18 @@ Object thedark "Darkness"
 			! the first sentence in the input  has been parsed
 			! and executed. Now remove it from parse so that
 			! the next sentence can be parsed
-			_copylength = 2 * _parsearraylength + 1;
-			for(_i = 1, _j = 2 * _sentencelength + 1: _j < _copylength: _i++, _j++)
+#Ifv5;
+			@log_shift _sentencelength 2 -> _i; ! Multiply by 4
+			_j = parse + 2;
+			_i = _i + _j;
+			_copylength = _sentencelength - _parsearraylength;
+			@log_shift _copylength 2 -> _copylength; ! Multiply by 4
+			@copy_table _i _j _copylength;
+#Ifnot;
+			_copylength = _parsearraylength + _parsearraylength;
+			for(_i = 1, _j = _sentencelength + _sentencelength + 1: _j <= _copylength: _i++, _j++)
 				parse-->_i = parse-->_j;
-
+#Endif;
 			parse->1 = _parsearraylength - _sentencelength;
 			_disallow_complex_again = true; ! cannot parse "x me.g.g.g"
 		} else {
