@@ -1,8 +1,8 @@
 /* ------------------------------------------------------------------------- */
 /*   "chars" : Character set mappings and the Z-machine alphabet table       */
 /*                                                                           */
-/*   Part of Inform 6.36                                                     */
-/*   copyright (c) Graham Nelson 1993 - 2021                                 */
+/*   Part of Inform 6.44                                                     */
+/*   copyright (c) Graham Nelson 1993 - 2025                                 */
 /*                                                                           */
 /* ------------------------------------------------------------------------- */
 /*  Inform uses six different character representations:                     */
@@ -322,8 +322,9 @@ static void read_source_to_iso_file(uchar *uccg)
 /*                                                                           */
 /*      00         remains 0 (meaning "end of file")                         */
 /*      TAB        becomes SPACE                                             */
+/*      0a         remains '\n'                                              */
 /*      0c         ("form feed") becomes '\n'                                */
-/*      0d         becomes '\n'                                              */
+/*      0d         remains '\r'                                              */
 /*      other control characters become '?'                                  */
 /*      7f         becomes '?'                                               */
 /*      80 to 9f   become '?'                                                */
@@ -346,7 +347,7 @@ static void make_source_to_iso_grid(void)
         for (n=1; n<32; n++) source_to_iso_grid[n] = '?';
         source_to_iso_grid[10] = '\n';
         source_to_iso_grid[12] = '\n';
-        source_to_iso_grid[13] = '\n';
+        source_to_iso_grid[13] = '\r';
         source_to_iso_grid[127] = '?';
         source_to_iso_grid[TAB_CHARACTER] = ' ';
 
@@ -1142,7 +1143,7 @@ extern int32 text_to_unicode(char *text)
         }
     }
 
-    if ((isdigit(text[1])) || (text[1] == '@'))
+    if ((isdigit((uchar)text[1])) || (text[1] == '@'))
     {   ebf_error("'@' plus an accent code or '@{...}'", text);
         textual_form_length = 1;
         return '?';
@@ -1245,13 +1246,13 @@ extern void change_character_set(void)
 extern void make_lower_case(char *str)
 {   int i;
     for (i=0; str[i]!=0; i++)
-        if ((((uchar)str[i])<128) && (isupper(str[i]))) str[i]=tolower(str[i]);
+        if ((((uchar)str[i])<128) && (isupper((uchar)str[i]))) str[i]=tolower((uchar)str[i]);
 }
 
 extern void make_upper_case(char *str)
 {   int i;
     for (i=0; str[i]!=0; i++)
-        if ((((uchar)str[i])<128) && (islower(str[i]))) str[i]=toupper(str[i]);
+        if ((((uchar)str[i])<128) && (islower((uchar)str[i]))) str[i]=toupper((uchar)str[i]);
 }
 
 /* ========================================================================= */
