@@ -1,8 +1,8 @@
 /* ------------------------------------------------------------------------- */
 /*   "lexer" : Lexical analyser                                              */
 /*                                                                           */
-/*   Part of Inform 6.42                                                     */
-/*   copyright (c) Graham Nelson 1993 - 2024                                 */
+/*   Part of Inform 6.44                                                     */
+/*   copyright (c) Graham Nelson 1993 - 2025                                 */
 /*                                                                           */
 /* ------------------------------------------------------------------------- */
 
@@ -178,7 +178,7 @@ extern debug_locations get_token_location_end
        prior block and last exactly zero bytes there.  It's misleading to
        include such ranges, so we gobble them. */
     if (beginning.head->location.end_byte_index ==
-          beginning.beginning_byte_index &&
+        beginning.beginning_byte_index &&
         beginning.head->next)
     {   beginning.head = beginning.head->next;
         result.location = beginning.head->location;
@@ -473,8 +473,8 @@ static char *opcode_list_g[] = {
 };
 
 keyword_group opcode_macros =
-{ { "" },
-  OPCODE_MACRO_TT, FALSE, TRUE
+{   { "" },
+    OPCODE_MACRO_TT, FALSE, TRUE
 };
 
 static char *opmacro_list_z[] = { "" };
@@ -577,6 +577,7 @@ keyword_group system_constants =
     "lowest_object_number", "highest_object_number",
     "oddeven_packing",
     "grammar_table", "dictionary_table", "dynam_string_table",
+    "highest_meta_action_number",
     "" },
     SYSTEM_CONSTANT_TT, FALSE, TRUE
 };
@@ -783,8 +784,8 @@ extern void construct_local_variable_tables(void)
         local_variables.keywords[i] = p;
         if (p[1] == 0)
         {   one_letter_locals[(uchar)p[0]] = i;
-            if (isupper(p[0])) one_letter_locals[tolower(p[0])] = i;
-            if (islower(p[0])) one_letter_locals[toupper(p[0])] = i;
+            if (isupper((uchar)p[0])) one_letter_locals[tolower((uchar)p[0])] = i;
+            if (islower((uchar)p[0])) one_letter_locals[toupper((uchar)p[0])] = i;
         }
         h = hash_code_from_string(p);
         if (local_variable_hash_table[h] == -1)
@@ -1729,9 +1730,10 @@ extern void release_token_texts(void)
 }
 
 extern void put_token_back(void)
-{   tokens_put_back++;
+{   int pos;
 
-    int pos = circle_position - tokens_put_back + 1;
+    tokens_put_back++;
+    pos = circle_position - tokens_put_back + 1;
     if (pos<0) pos += CIRCLE_SIZE;
 
     if (tokens_trace_level > 0)
