@@ -1,8 +1,8 @@
 /* ------------------------------------------------------------------------- */
 /*   "symbols" :  The symbols table; creating stock of reserved words        */
 /*                                                                           */
-/*   Part of Inform 6.41                                                     */
-/*   copyright (c) Graham Nelson 1993 - 2022                                 */
+/*   Part of Inform 6.44                                                     */
+/*   copyright (c) Graham Nelson 1993 - 2025                                 */
 /*                                                                           */
 /* ------------------------------------------------------------------------- */
 
@@ -56,10 +56,13 @@ symbolinfo *symbols;                           /* Allocated up to no_symbols */
 static memory_list symbols_memlist;
 symboldebuginfo *symbol_debug_info;            /* Allocated up to no_symbols */
 static memory_list symbol_debug_info_memlist;
+static char *temp_symbol_buf;        /* used in write_the_identifier_names() */
+static memory_list temp_symbol_buf_memlist;
 
 /* ------------------------------------------------------------------------- */
 /*   Memory to hold the text of symbol names: note that this memory is       */
-/*   allocated as needed in chunks of size SYMBOLS_CHUNK_SIZE.               */
+/*   allocated as needed in chunks of size SYMBOLS_CHUNK_SIZE. (Or           */
+/*   larger, if needed for a particularly enormous symbol.)                  */
 /* ------------------------------------------------------------------------- */
 
 #define SYMBOLS_CHUNK_SIZE (4096)
@@ -224,10 +227,16 @@ extern int get_symbol_index(char *p)
     return -1;
 }
 
-extern int symbol_index(char *p, int hashcode)
+extern int symbol_index(char *p, int hashcode, int *created)
 {
     /*  Return the index in the symbols array of symbol "p", creating a
-        new symbol with that name if it isn't already there.
+        new symbol with that name if it isn't already there. This
+        always returns a valid symbol index.
+
+        The optional created argument receives TRUE if the symbol
+        was newly created.
+
+        Pass in the hashcode of p if you know it, or -1 if you don't.
 
         New symbols are created with flag UNKNOWN_SFLAG, value 0x100
         (a 2-byte quantity in Z-machine terms) and type CONSTANT_T.
@@ -251,6 +260,7 @@ extern int symbol_index(char *p, int hashcode)
         {
             if (track_unused_routines)
                 df_note_function_symbol(this);
+            if (created) *created = FALSE;
             return this;
         }
         if (new_entry > 0) break;
@@ -260,7 +270,7 @@ extern int symbol_index(char *p, int hashcode)
     } while (this != -1);
 
     if (symdef_trace_setting)
-        printf("Encountered symbol %d '%s'\n", no_symbols, p);
+        printf("%s: Encountered symbol %d '%s'\n", current_location_text(), no_symbols, p);
     
     ensure_memory_list_available(&symbols_memlist, no_symbols+1);
     if (debugfile_switch)
@@ -276,18 +286,19 @@ extern int symbol_index(char *p, int hashcode)
     }
 
     len = strlen(p);
-    if (symbols_free_space+len+1 >= symbols_ceiling)
-    {   symbols_free_space
-            = my_malloc(SYMBOLS_CHUNK_SIZE, "symbol names chunk");
-        symbols_ceiling = symbols_free_space + SYMBOLS_CHUNK_SIZE;
+    if (!symbols_free_space || symbols_free_space+len+1 >= symbols_ceiling)
+    {
+        /* Allocate a new chunk whose size is big enough for the current
+           symbol, or SYMBOLS_CHUNK_SIZE, whichever is greater. */
+        int chunklen = SYMBOLS_CHUNK_SIZE;
+        if (chunklen < len+1)
+            chunklen = len+1;
+        symbols_free_space
+            = my_malloc(chunklen, "symbol names chunk");
+        symbols_ceiling = symbols_free_space + chunklen;
         ensure_memory_list_available(&symbol_name_space_chunks_memlist, no_symbol_name_space_chunks+1);
         symbol_name_space_chunks[no_symbol_name_space_chunks++]
             = symbols_free_space;
-        if (symbols_free_space+len+1 >= symbols_ceiling)
-        {
-            /* This should be impossible, since SYMBOLS_CHUNK_SIZE > MAX_IDENTIFIER_LENGTH. */
-            fatalerror("Symbol exceeds the maximum possible length");
-        }
     }
 
     strcpy(symbols_free_space, p);
@@ -309,17 +320,29 @@ extern int symbol_index(char *p, int hashcode)
 
     if (track_unused_routines)
         df_note_function_symbol(no_symbols);
+    if (created) *created = TRUE;
     return(no_symbols++);
 }
 
-extern void end_symbol_scope(int k)
+extern void end_symbol_scope(int k, int neveruse)
 {
     /* Remove the given symbol from the hash table, making it
-       invisible to symbol_index. This is used by the Undef directive.
-       If the symbol is not found, this silently does nothing.
+       invisible to symbol_index. This is used by the Undef directive
+       and put_token_back().
+
+       If you know the symbol has never been used, set neveruse and
+       it will be flagged as an error if it *is* used.
+       
+       If the symbol is not found in the hash table, this silently does
+       nothing.
     */
 
     int j;
+    
+    symbols[k].flags |= UNHASHED_SFLAG;
+    if (neveruse)
+        symbols[k].flags |= DISCARDED_SFLAG;
+        
     j = hash_code_from_string(symbols[k].name);
     if (start_of_list[j] == k)
     {   start_of_list[j] = symbols[k].next_entry;
@@ -375,8 +398,8 @@ static void describe_flags(int flags)
     if (flags & USED_SFLAG)     printf("(used) ");
     if (flags & DEFCON_SFLAG)   printf("(Defaulted) ");
     if (flags & STUB_SFLAG)     printf("(Stubbed) ");
-    if (flags & IMPORT_SFLAG)   printf("(Imported) ");
-    if (flags & EXPORT_SFLAG)   printf("(Exported) ");
+    if (flags & UNHASHED_SFLAG) printf("(not in hash chain) ");
+    if (flags & DISCARDED_SFLAG)  printf("(removed, do not use) ");
     if (flags & ALIASED_SFLAG)  printf("(aliased) ");
     if (flags & CHANGE_SFLAG)   printf("(value will change) ");
     if (flags & SYSTEM_SFLAG)   printf("(System) ");
@@ -510,19 +533,27 @@ extern void issue_unused_warnings(void)
 
     /*  Update any ad-hoc variables that might help the library  */
     if (glulx_mode)
-    {   global_initial_value[10]=statusline_flag;
+    {
+        set_constant_otv(&global_initial_value[10], statusline_flag);
     }
     /*  Now back to mark anything necessary as used  */
 
-    i = symbol_index("Main", -1);
-    if (!(symbols[i].flags & UNKNOWN_SFLAG)) symbols[i].flags |= USED_SFLAG;
+    i = get_symbol_index("Main");
+    if (i >= 0 && !(symbols[i].flags & UNKNOWN_SFLAG)) {
+        symbols[i].flags |= USED_SFLAG;
+    }
 
     for (i=0;i<no_symbols;i++)
     {   if (((symbols[i].flags
-             & (SYSTEM_SFLAG + UNKNOWN_SFLAG + EXPORT_SFLAG
+             & (SYSTEM_SFLAG + UNKNOWN_SFLAG
                 + INSF_SFLAG + USED_SFLAG + REPLACE_SFLAG)) == 0)
-             && (symbols[i].type != OBJECT_T))
+             && (symbols[i].type != OBJECT_T)) {
             dbnu_warning(typename(symbols[i].type), symbols[i].name, symbols[i].line);
+        }
+        if ((symbols[i].flags & DISCARDED_SFLAG)
+            && (symbols[i].flags & USED_SFLAG)) {
+            error_named_at("Symbol was removed from the symbol table, but seems to be in use anyway", symbols[i].name, symbols[i].line);
+        }
     }
 }
 
@@ -539,18 +570,18 @@ extern void issue_debug_symbol_warnings(void)
 
 /* ------------------------------------------------------------------------- */
 /*   These are arrays used only during story file creation, and not          */
-/*   allocated until then.                                                   */
+/*   allocated until just before write_the_identifier_names() time.          */
 
        int32 *individual_name_strings; /* Packed addresses of Z-encoded
                                           strings of the names of the
                                           properties: this is an array
                                           indexed by the property ID         */
-       int32 *action_name_strings;     /* Ditto for actions                  */
+       int32 *action_name_strings;     /* Ditto for actions and fake actions */
        int32 *attribute_name_strings;  /* Ditto for attributes               */
        int32 *array_name_strings;      /* Ditto for arrays                   */
 
 extern void write_the_identifier_names(void)
-{   int i, j, k, t, null_value; char idname_string[256];
+{   int i, j, k, t, null_value;
     static char unknown_attribute[20] = "<unknown attribute>";
 
     for (i=0; i<no_individual_properties; i++)
@@ -566,117 +597,155 @@ extern void write_the_identifier_names(void)
         if ((t == INDIVIDUAL_PROPERTY_T) || (t == PROPERTY_T))
         {   if (symbols[i].flags & ALIASED_SFLAG)
             {   if (individual_name_strings[symbols[i].value] == 0)
-                {   sprintf(idname_string, "%s", symbols[i].name);
+                {
+                    int sleni = strlen(symbols[i].name);
+                    ensure_memory_list_available(&temp_symbol_buf_memlist, sleni+1);
+                    sprintf(temp_symbol_buf, "%s", symbols[i].name);
 
                     for (j=i+1, k=0; (j<no_symbols && k<3); j++)
                     {   if ((symbols[j].type == symbols[i].type)
                             && (symbols[j].value == symbols[i].value))
-                        {   sprintf(idname_string+strlen(idname_string),
+                        {
+                            int slenj = strlen(symbols[j].name);
+                            ensure_memory_list_available(&temp_symbol_buf_memlist, strlen(temp_symbol_buf)+1+slenj+1);
+                            sprintf(temp_symbol_buf+strlen(temp_symbol_buf),
                                 "/%s", symbols[j].name);
                             k++;
                         }
                     }
 
                     individual_name_strings[symbols[i].value]
-                        = compile_string(idname_string, STRCTX_SYMBOL);
+                        = compile_string(temp_symbol_buf, STRCTX_SYMBOL);
                 }
             }
             else
-            {   sprintf(idname_string, "%s", symbols[i].name);
-
+            {
                 individual_name_strings[symbols[i].value]
-                    = compile_string(idname_string, STRCTX_SYMBOL);
+                    = compile_string(symbols[i].name, STRCTX_SYMBOL);
             }
         }
         if (t == ATTRIBUTE_T)
-        {   if (symbols[i].flags & ALIASED_SFLAG)
+        {
+            if (symbols[i].flags & ALIASED_SFLAG)
             {   if (attribute_name_strings[symbols[i].value] == null_value)
-                {   sprintf(idname_string, "%s", symbols[i].name);
+                {
+                    int sleni = strlen(symbols[i].name);
+                    ensure_memory_list_available(&temp_symbol_buf_memlist, sleni+1);
+                    sprintf(temp_symbol_buf, "%s", symbols[i].name);
 
                     for (j=i+1, k=0; (j<no_symbols && k<3); j++)
                     {   if ((symbols[j].type == symbols[i].type)
                             && (symbols[j].value == symbols[i].value))
-                        {   sprintf(idname_string+strlen(idname_string),
+                        {
+                            int slenj = strlen(symbols[j].name);
+                            ensure_memory_list_available(&temp_symbol_buf_memlist, strlen(temp_symbol_buf)+1+slenj+1);
+                            sprintf(temp_symbol_buf+strlen(temp_symbol_buf),
                                 "/%s", symbols[j].name);
                             k++;
                         }
                     }
 
                     attribute_name_strings[symbols[i].value]
-                        = compile_string(idname_string, STRCTX_SYMBOL);
+                        = compile_string(temp_symbol_buf, STRCTX_SYMBOL);
                 }
             }
             else
-            {   sprintf(idname_string, "%s", symbols[i].name);
-
+            {
                 attribute_name_strings[symbols[i].value]
-                    = compile_string(idname_string, STRCTX_SYMBOL);
+                    = compile_string(symbols[i].name, STRCTX_SYMBOL);
             }
         }
+        
         if (symbols[i].flags & ACTION_SFLAG)
-        {   sprintf(idname_string, "%s", symbols[i].name);
-            idname_string[strlen(idname_string)-3] = 0;
-
-            if (debugfile_switch)
-            {   debug_file_printf("<action>");
-                debug_file_printf
-                    ("<identifier>##%s</identifier>", idname_string);
-                debug_file_printf("<value>%d</value>", symbols[i].value);
-                debug_file_printf("</action>");
-            }
+        {
+            int sleni = strlen(symbols[i].name);
+            ensure_memory_list_available(&temp_symbol_buf_memlist, sleni+1);
+            sprintf(temp_symbol_buf, "%s", symbols[i].name);
+            temp_symbol_buf[strlen(temp_symbol_buf)-3] = 0;
 
             action_name_strings[symbols[i].value]
-                = compile_string(idname_string, STRCTX_SYMBOL);
+                = compile_string(temp_symbol_buf, STRCTX_SYMBOL);
         }
     }
 
     for (i=0; i<no_symbols; i++)
     {   if (symbols[i].type == FAKE_ACTION_T)
-        {   sprintf(idname_string, "%s", symbols[i].name);
-            idname_string[strlen(idname_string)-3] = 0;
+        {
+            int sleni = strlen(symbols[i].name);
+            ensure_memory_list_available(&temp_symbol_buf_memlist, sleni+1);
+            sprintf(temp_symbol_buf, "%s", symbols[i].name);
+            temp_symbol_buf[strlen(temp_symbol_buf)-3] = 0;
 
             action_name_strings[symbols[i].value
-                    - ((grammar_version_number==1)?256:4096) + no_actions]
-                = compile_string(idname_string, STRCTX_SYMBOL);
+                    - lowest_fake_action() + no_actions]
+                = compile_string(temp_symbol_buf, STRCTX_SYMBOL);
         }
     }
 
     for (j=0; j<no_arrays; j++)
-    {   i = arrays[j].symbol;
-        sprintf(idname_string, "%s", symbols[i].name);
-
+    {
+        i = arrays[j].symbol;
         array_name_strings[j]
-            = compile_string(idname_string, STRCTX_SYMBOL);
+            = compile_string(symbols[i].name, STRCTX_SYMBOL);
     }
-  if (define_INFIX_switch)
-  { for (i=0; i<no_symbols; i++)
-    {   if (symbols[i].type == GLOBAL_VARIABLE_T)
-        {   sprintf(idname_string, "%s", symbols[i].name);
-            array_name_strings[no_arrays + symbols[i].value -16]
-                = compile_string(idname_string, STRCTX_SYMBOL);
+    
+    if (define_INFIX_switch)
+    {
+        for (i=0; i<no_symbols; i++)
+        {   if (symbols[i].type == GLOBAL_VARIABLE_T)
+            {
+                array_name_strings[no_arrays + symbols[i].value -16]
+                    = compile_string(symbols[i].name, STRCTX_SYMBOL);
+            }
         }
-    }
-
-    for (i=0; i<no_named_routines; i++)
-    {   sprintf(idname_string, "%s", symbols[named_routine_symbols[i]].name);
+        
+        for (i=0; i<no_named_routines; i++)
+        {
             array_name_strings[no_arrays + no_globals + i]
-                = compile_string(idname_string, STRCTX_SYMBOL);
-    }
-
-    for (i=0, no_named_constants=0; i<no_symbols; i++)
-    {   if (((symbols[i].type == OBJECT_T) || (symbols[i].type == CLASS_T)
-            || (symbols[i].type == CONSTANT_T))
-            && ((symbols[i].flags & (UNKNOWN_SFLAG+ACTION_SFLAG))==0))
-        {   sprintf(idname_string, "%s", symbols[i].name);
-            array_name_strings[no_arrays + no_globals + no_named_routines
-                + no_named_constants++]
-                = compile_string(idname_string, STRCTX_SYMBOL);
+                = compile_string(symbols[named_routine_symbols[i]].name, STRCTX_SYMBOL);
+        }
+        
+        for (i=0, no_named_constants=0; i<no_symbols; i++)
+        {   if (((symbols[i].type == OBJECT_T) || (symbols[i].type == CLASS_T)
+                 || (symbols[i].type == CONSTANT_T))
+                && ((symbols[i].flags & (UNKNOWN_SFLAG+ACTION_SFLAG))==0))
+            {
+                array_name_strings[no_arrays + no_globals + no_named_routines
+                                   + no_named_constants++]
+                    = compile_string(symbols[i].name, STRCTX_SYMBOL);
+            }
         }
     }
-  }
 
     veneer_mode = FALSE;
 }
+
+extern void write_debug_information_for_actions(void)
+{
+    int i, action;
+    
+    for (i=0; i<no_symbols; i++) {
+        if (symbols[i].flags & ACTION_SFLAG)
+        {
+            int sleni = strlen(symbols[i].name);
+            ensure_memory_list_available(&temp_symbol_buf_memlist, sleni+1);
+            sprintf(temp_symbol_buf, "%s", symbols[i].name);
+            temp_symbol_buf[strlen(temp_symbol_buf)-3] = 0;
+
+            action = symbols[i].value;
+            if (GRAMMAR_META_FLAG) {
+                action = sorted_actions[action].internal_to_ext;
+            }
+            
+            debug_file_printf("<action>");
+            debug_file_printf
+                ("<identifier>##%s</identifier>", temp_symbol_buf);
+            debug_file_printf("<value>%d</value>", action);
+            debug_file_printf("</action>");
+        }
+    }
+}
+
 /* ------------------------------------------------------------------------- */
 /*   Creating symbols                                                        */
 /* ------------------------------------------------------------------------- */
@@ -696,7 +765,7 @@ extern void assign_symbol(int index, int32 value, int type)
     assign_symbol_base(index, value, type);
     symbols[index].marker = 0;
     if (symdef_trace_setting)
-        printf("Defined symbol %d '%s' as %d (%s)\n", index, symbols[index].name, value, typename(type));
+        printf("%s: Defined symbol %d '%s' as %d (%s)\n", current_location_text(), index, symbols[index].name, value, typename(type));
 }
 
 extern void assign_marked_symbol(int index, int marker, int32 value, int type)
@@ -704,7 +773,7 @@ extern void assign_marked_symbol(int index, int marker, int32 value, int type)
     assign_symbol_base(index, value, type);
     symbols[index].marker = marker;
     if (symdef_trace_setting)
-        printf("Defined symbol %d '%s' as %s %d (%s)\n", index, symbols[index].name, describe_mv(marker), value, typename(type));
+        printf("%s: Defined symbol %d '%s' as %s %d (%s)\n", current_location_text(), index, symbols[index].name, describe_mv(marker), value, typename(type));
 }
 
 static void emit_debug_information_for_predefined_symbol
@@ -756,7 +825,8 @@ static void emit_debug_information_for_predefined_symbol
 }
 
 static void create_symbol(char *p, int32 value, int type)
-{   int i = symbol_index(p, -1);
+{
+    int i = symbol_index(p, -1, NULL);
     if (!(symbols[i].flags & (UNKNOWN_SFLAG + REDEFINABLE_SFLAG))) {
         /* Symbol already defined! */
         if (symbols[i].value == value && symbols[i].type == type) {
@@ -776,9 +846,13 @@ static void create_symbol(char *p, int32 value, int type)
 }
 
 static void create_rsymbol(char *p, int value, int type)
-{   int i = symbol_index(p, -1);
-    /* This is only called for a few symbols with known names.
-       They will not collide. */
+{
+    /* Create a symbol which the user can redefine later.
+       This is only called for a few symbols with known names;
+       they shouldn't collide. */
+    int i = symbol_index(p, -1, NULL);
+    if (!(symbols[i].flags & UNKNOWN_SFLAG))
+        compiler_error_named("create_rsymbol called with existing symbol", p);
     symbols[i].value = value; symbols[i].type = type; symbols[i].line = blank_brief_location;
     symbols[i].flags = USED_SFLAG + SYSTEM_SFLAG + REDEFINABLE_SFLAG;
     emit_debug_information_for_predefined_symbol(p, i, value, type);
@@ -797,12 +871,10 @@ static void stockup_symbols(void)
     create_symbol("true",           1, CONSTANT_T);
     create_symbol("false",          0, CONSTANT_T);
 
-    /* Glulx defaults to GV2; Z-code to GV1 */
-    if (!glulx_mode)
-        create_rsymbol("Grammar__Version", 1, CONSTANT_T);
-    else
-        create_rsymbol("Grammar__Version", 2, CONSTANT_T);
-    grammar_version_symbol = symbol_index("Grammar__Version", -1);
+    /* Glulx defaults to GV2; Z-code to GV1. These may be modified by
+       command-line options, but we haven't applied that change yet. */
+    create_rsymbol("Grammar__Version", grammar_version_number, CONSTANT_T);
+    grammar_version_symbol = get_symbol_index("Grammar__Version");
 
     if (runtime_error_checking_switch)
         create_rsymbol("STRICT_MODE",0, CONSTANT_T);
@@ -815,6 +887,12 @@ static void stockup_symbols(void)
         create_symbol("infix__watching", 0, ATTRIBUTE_T);
     }
 
+    if (OMIT_SYMBOL_TABLE)
+        create_symbol("OMIT_SYMBOL_TABLE", 0, CONSTANT_T);
+
+    if (GRAMMAR_META_FLAG)
+        create_symbol("GRAMMAR_META_FLAG", 0, CONSTANT_T);
+
     create_symbol("WORDSIZE",        WORDSIZE, CONSTANT_T);
     /* DICT_ENTRY_BYTES must be REDEFINABLE_SFLAG because the Version directive can change it. */
     create_rsymbol("DICT_ENTRY_BYTES", DICT_ENTRY_BYTE_LENGTH, CONSTANT_T);
@@ -840,17 +918,19 @@ static void stockup_symbols(void)
     }    
 
     if (!glulx_mode) {
-        create_symbol("temp_global",  255, GLOBAL_VARIABLE_T);
-        create_symbol("temp__global2", 254, GLOBAL_VARIABLE_T);
-        create_symbol("temp__global3", 253, GLOBAL_VARIABLE_T);
-        create_symbol("temp__global4", 252, GLOBAL_VARIABLE_T);
-        create_symbol("self",         251, GLOBAL_VARIABLE_T);
-        create_symbol("sender",       250, GLOBAL_VARIABLE_T);
-        create_symbol("sw__var",      249, GLOBAL_VARIABLE_T);
-        
-        create_symbol("sys__glob0",     16, GLOBAL_VARIABLE_T);
-        create_symbol("sys__glob1",     17, GLOBAL_VARIABLE_T);
-        create_symbol("sys__glob2",     18, GLOBAL_VARIABLE_T);
+        create_symbol("temp_global",   globalv_z_temp_var1, GLOBAL_VARIABLE_T);
+        create_symbol("temp__global2", globalv_z_temp_var2, GLOBAL_VARIABLE_T);
+        create_symbol("temp__global3", globalv_z_temp_var3, GLOBAL_VARIABLE_T);
+        create_symbol("temp__global4", globalv_z_temp_var4, GLOBAL_VARIABLE_T);
+        create_symbol("self",          globalv_z_self, GLOBAL_VARIABLE_T);
+        create_symbol("sender",        globalv_z_sender, GLOBAL_VARIABLE_T);
+        create_symbol("sw__var",       globalv_z_sw__var, GLOBAL_VARIABLE_T);
+
+        /* These three are always the first three Z-machine globals,
+           which are significant in v3 (but not any later version). */
+        create_symbol("sys__glob0",    16, GLOBAL_VARIABLE_T);
+        create_symbol("sys__glob1",    17, GLOBAL_VARIABLE_T);
+        create_symbol("sys__glob2",    18, GLOBAL_VARIABLE_T);
         
         create_symbol("create",        64, INDIVIDUAL_PROPERTY_T);
         create_symbol("recreate",      65, INDIVIDUAL_PROPERTY_T);
@@ -865,50 +945,50 @@ static void stockup_symbols(void)
         /* In Glulx, these system globals are entered in order, not down 
            from 255. */
         create_symbol("temp_global",  MAX_LOCAL_VARIABLES+0, 
-          GLOBAL_VARIABLE_T);
+            GLOBAL_VARIABLE_T);
         create_symbol("temp__global2", MAX_LOCAL_VARIABLES+1, 
-          GLOBAL_VARIABLE_T);
+            GLOBAL_VARIABLE_T);
         create_symbol("temp__global3", MAX_LOCAL_VARIABLES+2, 
-          GLOBAL_VARIABLE_T);
+            GLOBAL_VARIABLE_T);
         create_symbol("temp__global4", MAX_LOCAL_VARIABLES+3, 
-          GLOBAL_VARIABLE_T);
+            GLOBAL_VARIABLE_T);
         create_symbol("self",         MAX_LOCAL_VARIABLES+4, 
-          GLOBAL_VARIABLE_T);
+            GLOBAL_VARIABLE_T);
         create_symbol("sender",       MAX_LOCAL_VARIABLES+5, 
-          GLOBAL_VARIABLE_T);
+            GLOBAL_VARIABLE_T);
         create_symbol("sw__var",      MAX_LOCAL_VARIABLES+6, 
-          GLOBAL_VARIABLE_T);
+            GLOBAL_VARIABLE_T);
 
         /* These are almost certainly meaningless, and can be removed. */
         create_symbol("sys__glob0",     MAX_LOCAL_VARIABLES+7, 
-          GLOBAL_VARIABLE_T);
+            GLOBAL_VARIABLE_T);
         create_symbol("sys__glob1",     MAX_LOCAL_VARIABLES+8, 
-          GLOBAL_VARIABLE_T);
+            GLOBAL_VARIABLE_T);
         create_symbol("sys__glob2",     MAX_LOCAL_VARIABLES+9, 
-          GLOBAL_VARIABLE_T);
+            GLOBAL_VARIABLE_T);
 
         /* value of statusline_flag to be written later */
         create_symbol("sys_statusline_flag",  MAX_LOCAL_VARIABLES+10, 
-          GLOBAL_VARIABLE_T);
+            GLOBAL_VARIABLE_T);
 
         /* These are created in order, but not necessarily at a fixed
            value. */
         create_symbol("create",        INDIV_PROP_START+0, 
-          INDIVIDUAL_PROPERTY_T);
+            INDIVIDUAL_PROPERTY_T);
         create_symbol("recreate",      INDIV_PROP_START+1, 
-          INDIVIDUAL_PROPERTY_T);
+            INDIVIDUAL_PROPERTY_T);
         create_symbol("destroy",       INDIV_PROP_START+2, 
-          INDIVIDUAL_PROPERTY_T);
+            INDIVIDUAL_PROPERTY_T);
         create_symbol("remaining",     INDIV_PROP_START+3, 
-          INDIVIDUAL_PROPERTY_T);
+            INDIVIDUAL_PROPERTY_T);
         create_symbol("copy",          INDIV_PROP_START+4, 
-          INDIVIDUAL_PROPERTY_T);
+            INDIVIDUAL_PROPERTY_T);
         create_symbol("call",          INDIV_PROP_START+5, 
-          INDIVIDUAL_PROPERTY_T);
+            INDIVIDUAL_PROPERTY_T);
         create_symbol("print",         INDIV_PROP_START+6, 
-          INDIVIDUAL_PROPERTY_T);
+            INDIVIDUAL_PROPERTY_T);
         create_symbol("print_to_array",INDIV_PROP_START+7, 
-          INDIVIDUAL_PROPERTY_T);
+            INDIVIDUAL_PROPERTY_T);
 
         /* Floating-point constants. Note that FLOAT_NINFINITY is not
            -FLOAT_INFINITY, because float negation doesn't work that
@@ -1066,8 +1146,8 @@ static int df_functions_sorted_count;
 static df_reference_t **df_symbol_map;
 
 /* Globals used while a function is being compiled. When a function
-  *isn't* being compiled, df_current_function_addr will be DF_NOT_IN_FUNCTION
-  and df_current_function will refer to the global namespace record. */
+   *isn't* being compiled, df_current_function_addr will be DF_NOT_IN_FUNCTION
+   and df_current_function will refer to the global namespace record. */
 static df_function_t *df_current_function;
 static char *df_current_function_name;
 static uint32 df_current_function_addr;
@@ -1197,8 +1277,10 @@ extern void df_note_function_symbol(int symbol)
        we're in global scope, in which case it might be slower.
        (I suppose we could cache the df_function_t pointer of the
        current function, to speed things up.) */
-    if (!df_current_function || df_current_function_addr != df_current_function->address)
+    if (!df_current_function || df_current_function_addr != df_current_function->address) {
         compiler_error("DF: df_current_function does not match current address.");
+        return;
+    }
     ent->refsnext = df_current_function->refs;
     df_current_function->refs = ent;
 }
@@ -1223,15 +1305,15 @@ extern void locate_dead_functions(void)
        issue_unused_warnings(). But for the sake of thoroughness,
        we'll mark them specially. */
 
-    ix = symbol_index("Main__", -1);
-    if (symbols[ix].type == ROUTINE_T) {
+    ix = get_symbol_index("Main__");
+    if (ix >= 0 && symbols[ix].type == ROUTINE_T) {
         uint32 addr = symbols[ix].value * (glulx_mode ? 1 : scale_factor);
         tofunc = df_function_for_address(addr);
         if (tofunc)
             tofunc->usage |= DF_USAGE_MAIN;
     }
-    ix = symbol_index("Main", -1);
-    if (symbols[ix].type == ROUTINE_T) {
+    ix = get_symbol_index("Main");
+    if (ix >= 0 && symbols[ix].type == ROUTINE_T) {
         uint32 addr = symbols[ix].value * (glulx_mode ? 1 : scale_factor);
         tofunc = df_function_for_address(addr);
         if (tofunc)
@@ -1494,10 +1576,14 @@ uint32 df_stripped_offset_for_code_offset(uint32 offset, int *stripped)
 extern void df_prepare_function_iterate(void)
 {
     df_iterator = df_functions_head;
-    if (!df_iterator || df_iterator->address != DF_NOT_IN_FUNCTION)
+    if (!df_iterator || df_iterator->address != DF_NOT_IN_FUNCTION) {
         compiler_error("DF: Global namespace entry is not at the head of the chain.");
-    if (!df_iterator->funcnext || df_iterator->funcnext->address != 0)
+        return;
+    }
+    if (!df_iterator->funcnext || df_iterator->funcnext->address != 0) {
         compiler_error("DF: First function entry is not second in the chain.");
+        return;
+    }
 }
 
 /* This returns the end of the next function, and whether the next function
@@ -1524,11 +1610,12 @@ extern void init_symbols_vars(void)
     symbols = NULL;
     start_of_list = NULL;
     symbol_debug_info = NULL;
+    temp_symbol_buf = NULL;
 
     symbol_name_space_chunks = NULL;
     no_symbol_name_space_chunks = 0;
-    symbols_free_space=NULL;
-    symbols_ceiling=NULL;
+    symbols_free_space = NULL;
+    symbols_ceiling = NULL;
 
     no_symbols = 0;
 
@@ -1568,6 +1655,11 @@ extern void symbols_allocate_arrays(void)
             sizeof(symboldebuginfo), 6400, (void**)&symbol_debug_info,
             "symbol debug backpatch info");
     }
+    
+    initialise_memory_list(&temp_symbol_buf_memlist,
+        sizeof(char), 64, (void**)&temp_symbol_buf,
+        "temporary symbol name");
+        
     start_of_list = my_calloc(sizeof(int32), HASH_TAB_SIZE,
                      "hash code list beginnings");
 
@@ -1579,10 +1671,8 @@ extern void symbols_allocate_arrays(void)
         df_tables_closed = FALSE;
 
         df_symbol_map = my_calloc(sizeof(df_reference_t *), DF_SYMBOL_HASH_BUCKETS, "df symbol-map hash table");
-        memset(df_symbol_map, 0, sizeof(df_reference_t *) * DF_SYMBOL_HASH_BUCKETS);
 
         df_functions = my_calloc(sizeof(df_function_t *), DF_FUNCTION_HASH_BUCKETS, "df function hash table");
-        memset(df_functions, 0, sizeof(df_function_t *) * DF_FUNCTION_HASH_BUCKETS);
         df_functions_head = NULL;
         df_functions_tail = NULL;
 
@@ -1621,6 +1711,8 @@ extern void symbols_free_arrays(void)
     {
         deallocate_memory_list(&symbol_debug_info_memlist);
     }
+    deallocate_memory_list(&temp_symbol_buf_memlist);
+    
     my_free(&start_of_list, "hash code list beginnings");
 
     if (symbol_replacements)
